Package: CatPredi
Title: Optimal Categorisation of Continuous Variables in Prediction
        Models
Version: 2.0
Authors@R: c(
    person("Irantzu Barrio", email = "irantzu.barrio@ehu.eus", role = c("aut", "cre")),
    person("Maria Xose Rodriguez-Alvarez", role = "aut"),
    person("Inmaculada Arostegui", role = "ctb"),
    person("Diana Marcela Perez", role = "ctb"))
Description: Allows the user to categorise a continuous predictor variable in a logistic or a Cox proportional hazards regression setting, by maximising the discriminative ability of the model. I Barrio, I Arostegui, MX Rodriguez-Alvarez, JM Quintana (2015) <doi:10.1177/0962280215601873>. I Barrio,  MX Rodriguez-Alvarez, L Meira-Machado, C Esteban, I Arostegui (2017) <https://www.idescat.cat/sort/sort411/41.1.3.barrio-etal.pdf>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: CPE, doParallel, foreach, ggplot2, graphics, mgcv, rgenoud,
        rms, stats, survival
NeedsCompilation: no
Packaged: 2026-05-06 09:48:40 UTC; leire
Author: Irantzu Barrio [aut, cre],
  Maria Xose Rodriguez-Alvarez [aut],
  Inmaculada Arostegui [ctb],
  Diana Marcela Perez [ctb]
Maintainer: Irantzu Barrio <irantzu.barrio@ehu.eus>
Repository: CRAN
Date/Publication: 2026-05-08 07:42:31 UTC
Built: R 4.6.0; ; 2026-05-08 10:30:36 UTC; unix

Package: CatPredi
Title: Optimal Categorisation of Continuous Variables in Prediction
        Models
Version: 2.0
Authors@R: c(
    person("Irantzu Barrio", email = "irantzu.barrio@ehu.eus", role = c("aut", "cre")),
    person("Maria Xose Rodriguez-Alvarez", role = "aut"),
    person("Inmaculada Arostegui", role = "ctb"),
    person("Diana Marcela Perez", role = "ctb"))
Description: Allows the user to categorise a continuous predictor variable in a logistic or a Cox proportional hazards regression setting, by maximising the discriminative ability of the model. I Barrio, I Arostegui, MX Rodriguez-Alvarez, JM Quintana (2015) <doi:10.1177/0962280215601873>. I Barrio,  MX Rodriguez-Alvarez, L Meira-Machado, C Esteban, I Arostegui (2017) <https://www.idescat.cat/sort/sort411/41.1.3.barrio-etal.pdf>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: CPE, doParallel, foreach, ggplot2, graphics, mgcv, rgenoud,
        rms, stats, survival
NeedsCompilation: no
Packaged: 2026-05-06 09:48:40 UTC; leire
Author: Irantzu Barrio [aut, cre],
  Maria Xose Rodriguez-Alvarez [aut],
  Inmaculada Arostegui [ctb],
  Diana Marcela Perez [ctb]
Maintainer: Irantzu Barrio <irantzu.barrio@ehu.eus>
Repository: CRAN
Date/Publication: 2026-05-08 07:42:31 UTC
Built: R 4.5.2; ; 2026-05-08 11:07:48 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,catpredi)
S3method(plot,catpredi.survival)
S3method(print,catpredi)
S3method(print,catpredi.survival)
S3method(print,comp.cutpoints)
S3method(print,comp.cutpoints.binary)
S3method(print,comp.cutpoints.survival)
S3method(print,summary.catpredi)
S3method(print,summary.catpredi.survival)
S3method(summary,catpredi)
S3method(summary,catpredi.survival)
export(catpredi)
export(catpredi.survival)
export(comp.cutpoints)
export(comp.cutpoints.survival)
export(compare.AUC.ht)
export(controlcatpredi)
export(controlcatpredi.survival)
importFrom(foreach,"%dopar%")

#' Control function
#'
#' @param min.p.cat Set the minimun number of individuals in each category
#' @param grid Grid size for the AddFor and BackAddFor algorithms
#' @param B Number of bootstrap replicates for the AUC bias correction procedure
#' @param eps An argument for the BackAddFor algorithm, indicating whether the
#'    improvement between iterations is considered significant
#' @param b.method Allows to specify whether the bootstrap resampling should be
#'    done considering or not the outcome variable.
#'    The option "ncoutcome" indicates that the data is resampled without taking
#'    into account the response variable, while "coutcome" indicates that the data
#'    is resampled in regard to the response variable
#' @param print.gen Corresponds to the argument print.level of the \code{\link[rgenoud]{genoud}}
#' function of the package \pkg{rgenoud}.
#'
#' @returns
#' A list with components for each of the possible arguments.
#'
#' @description Function used to set several parameters to control the selection of
#'    the optimal cut points in a logistic regression model
#'
#' @references Mebane Jr, W. R., & Sekhon, J. S. (2011). Genetic optimization using derivatives:
#'    the rgenoud package for R. \emph{Journal of Statistical Software} 42\bold{11}, 1-26.
#'
#' @author Irantzu Barrio, Maria Xose Rodriguez-Alvarez, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
#'
#' @seealso
#' \code{\link{catpredi}}
#'
#' @export
#'
controlcatpredi <-
function(min.p.cat = 1,
	grid = 100,
	B = 50,
	eps=0.001,
  b.method = c("ncoutcome","coutcome"),
  print.gen = 0){
	list(min.p.cat = min.p.cat, grid = grid, B = B, eps = eps, b.method = match.arg(b.method), print.gen = print.gen)}

bootstrap.sample <-
function(data, group, method = c("ncoutcome","coutcome")) {
  method <- match.arg(method)
	if(method == "coutcome"){
		res <- do.call("rbind", lapply(split(data, data[,group]), function(x) x[sample(nrow(x), replace=TRUE),]))
	} else {
		res <- data[sample(nrow(data), replace=TRUE),]
	}
  res
}

#' @export
print.catpredi.survival <-
function(x, digits = 4, ...) {
	cat("\nCall:\n")
	print(x$call)
	cat("\n\n*************************************************\n")
	method <- switch(x$method,
	"addfor" = cat("Addfor Search Algorithm"),
	"genetic" = cat("Genetic Search Algorithm"),
	"backaddfor" = cat("Backaddfor Search Algorithm"))
	conc.index <- switch(x$conc.index,
	"cindex" = cat("Concordance C-index"),
	"cpe" = cat("Concordance Probability Estimator - CPE"))
	cat("\n*************************************************\n\n")
	if(x$method == "addfor" & x$conc.index == "cindex") {
		if(x$correct.index == TRUE){
			cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
			Cindex  <- format(x$results$Cindex, digits = digits, justify = "left")
			Cindex.corrected <- format(x$results$Cindex.cor, digits = digits, justify = "left")
			p.table <- cbind(cutpoints, Cindex,c(rep("NA",length(x$results$cutpoints)-1),Cindex.corrected))
			dimnames(p.table) <- list(rep("", l = length(x$results$Cindex)), c("Optimal cutpoints", "Optimal C-index", "Corrected C-index"))
			print(p.table, quote = FALSE, justify = "right")
		} else {
			cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
			Cindex  <- format(x$results$Cindex, digits = digits, justify = "left")
			p.table <- cbind(cutpoints, Cindex)
			dimnames(p.table) <- list(rep("", l = length(x$results$Cindex)), c("Optimal cutpoints", "Optimal C-index"))
			print(p.table, quote = FALSE, justify = "right")
		}
	} else if(x$method == "addfor" & x$conc.index == "cpe") {
		if(x$correct.index == TRUE){
			cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
			CPE  <- format(x$results$CPE, digits = digits, justify = "left")
			CPE.corrected <- format(x$results$CPE.cor, digits = digits, justify = "left")
			p.table <- cbind(cutpoints, CPE,c(rep("NA",length(x$results$cutpoints)-1),CPE.corrected))
			dimnames(p.table) <- list(rep("", l = length(x$results$CPE)), c("Optimal cutpoints", "Optimal CPE", "Corrected CPE"))
			print(p.table, quote = FALSE, justify = "right")
		} else {
			cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
			CPE  <- format(x$results$CPE, digits = digits, justify = "left")
			p.table <- cbind(cutpoints, CPE)
			dimnames(p.table) <- list(rep("", l = length(x$results$CPE)), c("Optimal cutpoints", "Optimal CPE"))
			print(p.table, quote = FALSE, justify = "right")
		}
	} else if(x$method == "genetic" & x$conc.index == "cindex") {
		if(x$correct.index == TRUE){
		cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
			dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
			print(cutpoints, quote = FALSE, justify = "right")
			cat("\n")
			Cindex <- cbind(format(x$results$Cindex, digits = digits, justify = "right"))
			dimnames(Cindex) <- list(rep("", l = length(x$results$Cindex)), "Optimal Cindex")
			print(Cindex, quote = FALSE, justify = "right")
			cat("\n")
			Cindex.corrected <- cbind(format(x$results$Cindex.cor, digits = digits, justify = "right"))
			dimnames(Cindex.corrected) <- list(rep("", l = length(x$results$Cindex.cor)), "Corrected Cindex")
			print(Cindex.corrected, quote = FALSE, justify = "right")
		} else {
			cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
			dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
			print(cutpoints, quote = FALSE, justify = "right")
			cat("\n")
			Cindex <- cbind(format(x$results$Cindex, digits = digits, justify = "right"))
			dimnames(Cindex) <- list(rep("", l = length(x$results$Cindex)), "Optimal Cindex")
			print(Cindex, quote = FALSE, justify = "right")
		}
	} else {
		if(x$correct.index == TRUE) {
			cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
			dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
			print(cutpoints, quote = FALSE, justify = "right")
			cat("\n")
			CPE <- cbind(format(x$results$CPE, digits = digits, justify = "right"))
			dimnames(CPE) <- list(rep("", l = length(x$results$CPE)), "Optimal CPE")
			print(CPE, quote = FALSE, justify = "right")
			cat("\n")
			CPE.corrected <- cbind(format(x$results$CPE.cor, digits = digits, justify = "right"))
			dimnames(CPE.corrected) <- list(rep("", l = length(x$results$CPE.cor)), "Corrected CPE")
			print(CPE.corrected, quote = FALSE, justify = "right")
		} else {
			cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
			dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
			print(cutpoints, quote = FALSE, justify = "right")
			cat("\n")
			CPE <- cbind(format(x$results$CPE, digits = digits, justify = "right"))
			dimnames(CPE) <- list(rep("", l = length(x$results$CPE)), "Optimal CPE")
			print(CPE, quote = FALSE, justify = "right")
		}
	}
	invisible(x)
}

calculate.AUC <-
function(points, formula, cat.var, data.f, range, min.p.cat = 1) {
	var.names <- all.vars(formula)
	x <- data.f[,cat.var]
	y <- data.f[,var.names[1]]
	if (all(sapply(points, function(x, range){ res <- if(x >= range[1] & x <= range[2]){TRUE} else {FALSE}}, range = range))) {
		cutoffs <- sort(unique(c(max(x), min(x), points)))
		x.cut <- cut(x, cutoffs, include.lowest=TRUE,right=TRUE)
		if(length(levels(x.cut)) > 1 & all(table(x.cut)>min.p.cat)) {
			if(length(var.names) == 1) {
				ratio <- prop.table(table(x.cut[y==1]))/prop.table(table(x.cut[y==0]))
				ratio.s <- ratio[match(x.cut, names(ratio))]
				auc <- compute.empirical.AUC(ratio.s[y==1], ratio.s[y==0])
			} else {
				 data.f[,"x.cut_"] <- x.cut
				 formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
				 fit <- mgcv::gam(formula.n, family = stats::binomial, data = data.f)
				 auc <- compute.empirical.AUC(fit$fitted[y==1], fit$fitted[y==0])
			}
		} else {
			auc <- NA
		}
	} else {
		auc <- NA
	}
	auc
}

auc.opt.corrected <-
  function(formula, cat.var, data, c.points, AUC, B=50 ,  b.method = c("ncoutcome","coutcome")) {
    b.method <- match.arg(b.method)
    auc.boot <- auc.original <- vector(length=B)
    var.names <- all.vars(formula)
    X <- data[,cat.var]
    Y <- data[,var.names[1]]
    data.f <- data
    for (i in 1:B) {
      data.b <- bootstrap.sample(data, group = var.names[1], method = b.method)
      Y.b <- data.b[,var.names[1]]
      X.b <- data.b[,cat.var]
      sel.point = sort(unique(c(min(X, X.b, na.rm=TRUE), max(X, X.b , na.rm=TRUE), c.points)))
      x.cut = cut(X, sel.point, include.lowest = TRUE, right = TRUE)
      x.cut.boot <- cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
      if(length(levels(x.cut.boot)) == length(levels(x.cut)) & all(table(x.cut.boot)>1)){
        if(length(var.names) == 1) {
          # Bootstrap
          ratio <- prop.table(table(x.cut.boot[Y.b==1]))/prop.table(table(x.cut.boot[Y.b==0]))
          ratio.s <- ratio[match(x.cut.boot, names(ratio))]
          auc.boot[i] <- compute.empirical.AUC(ratio.s[Y.b==1], ratio.s[Y.b==0])
          # Original Sample
          ratio.s <- ratio[match(x.cut, names(ratio))]
          auc.original[i] <- compute.empirical.AUC(ratio.s[Y==1], ratio.s[Y==0])
        } else {
          # Bootstrap
          data.b[,"x.cut_"] <- x.cut.boot
          formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
          fit.b <- mgcv::gam(formula.n, family = stats::binomial, data = data.b)
          auc.boot[i] <- compute.empirical.AUC(fit.b$fitted[Y.b==1], fit.b$fitted[Y.b==0])
          # Original Sample
          data.f[,"x.cut_"] <- x.cut
          fit.o <- stats::predict(fit.b, newdata =data.f, type="response")
          auc.original[i] <- compute.empirical.AUC(fit.o[Y==1], fit.o[Y==0])
        }
      } else {
        auc.boot[i] <- NA
        auc.original[i] <- NA
      }
    }
    auc.corrected <- AUC - mean(abs(auc.original - auc.boot), na.rm=TRUE)
    auc.corrected
  }

k.points.max.auc <-
function(formula, cat.var, data, range, k = 1, l.s.points = 100, min.p.cat = 1) {

	points <- NULL
	mat <-  matrix(ncol=2,nrow=k)
	colnames(mat)<- c("point","auc")

	for (i in 1:k){
		aucs <- select.cutpoint.auc(formula = formula, cat.var = cat.var, data = data, range = range, points = points, l.s.points = l.s.points, min.p.cat = min.p.cat)
		pos <- which(aucs[,2]==max(aucs[,2],na.rm=TRUE))
		if(length(pos)> 1 & i > 1) {
		  if (aucs[pos[1],1] <= mat[i-1,1]) {
			  pos <- max(pos)
		  } else {
		  pos = min(pos)
		  }
		} else {
			  pos = pos[1]
		}
		mat[i,1] <- aucs[pos,1]
		mat[i,2] <- aucs[pos,2]
		points <- c(points,mat[i,1])
	}
	mat
}

#' Plot the optimal cut points.
#'
#' @param x An object of type catpredi.
#' @param ... Additional arguments to be passed on to other functions. Not yet implemented.
#'
#' @returns
#' This function returns the plot of the relationship between the predictor variable and the outcome.
#'
#' @description Plots the relationship between the predictor variable is aimed to
#'    categorise and the response variable based on a GAM model.
#'    Additionally, the optimal cut points obtained with the catpredi() function
#'    are drawn on the graph.
#'
#' @references  I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
#'    A new approach to categorising continuous variables in prediction models:
#'    proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.
#'
#'    I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number of
#'    categories of the lymph node ratio in cancer research: A bootstrap-based
#'    hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
#'
#' @author Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
#'
#' @seealso
#' \code{\link{catpredi}}
#'
#' @examples
#'  library(CatPredi)
#'  \dontrun{
#'  set.seed(127)
#'  #Simulate data
#'  n = 100
#'  #Predictor variable
#'  xh <- rnorm(n, mean = 0, sd = 1)
#'  xd <- rnorm(n, mean = 1.5, sd = 1)
#'  x <- c(xh, xd)
#'  #Response
#'  y <- c(rep(0,n), rep(1,n))
#'  # Data frame
#'  df <- data.frame(y = y, x = x)
#'
#'  # Select optimal cut points using the AddFor algorithm
#'  res.backaddfor <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
#'                             data = df, method = "backaddfor", range = NULL, correct.AUC = FALSE)
#'  # Plot
#'  plot(res.backaddfor)
#'  }
#' @export
plot.catpredi <-
function(x, ...){
  # Activate pause between plots
  old_ask <- graphics::par(ask = TRUE)

  # Ensure the setting is reset at the end
  on.exit(graphics::par(ask = old_ask))

  # Fit the model
  formula <- stats::update(x$formula, stats::as.formula(paste("~ . + s(", x$cat.var, ")", sep = "")))
  fit.gam <- mgcv::gam(formula, family = stats::binomial, data = x$data)
  pos <- length(attr(stats::terms.formula(formula, specials = c("s")),"specials")$s)
  plot(fit.gam, select = pos, shade = TRUE, ylab = paste0("f(",x$cat.var,")"), all.terms = TRUE)
  graphics::abline(v = x$results$cutpoints, lty = 2)

  # Plot the OR
  formula <- stats::update(x$formula, stats::as.formula(paste0("~ . +", x$cat.var,"_CatPredi")))
  fit.gam.cut <- mgcv::gam(formula, family = stats::binomial, data = x$data)
  x_CatPredi = unique(x$data[,paste0(x$cat.var,"_CatPredi")])
  sel.cuts <- paste0(x$cat.var,"_CatPredi",x_CatPredi[-1])
  ORs.xcut <- data.frame(x_CatPredi = x_CatPredi,
                         OR = exp(fit.gam.cut$coef)[c("(Intercept)",sel.cuts)],
                         CI_Lower = exp(stats::confint.default(fit.gam.cut))[c("(Intercept)",sel.cuts),1],
                         CI_Upper = exp(stats::confint.default(fit.gam.cut))[c("(Intercept)",sel.cuts),2])

  p <- ggplot2::ggplot(ORs.xcut, ggplot2::aes(x = x_CatPredi, y = OR)) +
    ggplot2::geom_point(size = 3, color = "blue") + # Plot the ORs
    ggplot2::geom_errorbar(ggplot2::aes(ymin = CI_Lower, ymax = CI_Upper), width = 0.2, color = "blue") + # Add error bars
    ggplot2::geom_hline(yintercept = 1, linetype = "dashed", color = "red") + # Reference line at OR = 1
    ggplot2::coord_flip() + # Flip coordinates for better readability
    ggplot2::labs(
      title = "Odds Ratios with 95% Confidence Intervals",
      x = paste0(x$cat.var,"_CatPredi"),
      y = "Odds Ratio (OR)"
    ) +
    ggplot2::theme_minimal()
  print(p)
}

calculate.CPE <-
function(point, formula, cat.var, data.f, range, min.p.cat ) {
	var.names <- c(all.vars(formula), cat.var)
	X <- data.f[,cat.var]
	if (all(sapply(point, function(X, range){ res <- if(X> range[1] & X<range[2]){TRUE} else {FALSE}}, range = range))) {
		cutoffs <- sort(unique(c(max(X), min(X), point)))
		x.cut <- cut(X, cutoffs, include.lowest=TRUE,right=TRUE)
		if(length(levels(x.cut)) > 1 & all(table(x.cut)>min.p.cat)) {
			data.f[,"x.cut_"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
			fit <- try(rms::cph(formula.n, data = data.f))
			#if(class(fit) == "try-error"){
			if("try-error" %in% class(fit)) {
				cpe <- NA
			} else {
			  # cpe <- coxcpe(fit, data.f)
				cpe <- CPE::phcpe2(coef = fit$coefficients, coef.var = fit$var, design = stats::model.matrix(fit, data = data.f))$CPE
			}
		} else {
			cpe <- NA
		}
	} else {
		cpe <- NA
	}
	cpe
}

cindex.opt.corrected <-
function(formula, cat.var, data , c.points, cindex , B ,  b.method = c("ncoutcome","coutcome")){
  b.method <- match.arg(b.method)
	cind.boot <- cind.original  <- vector(length=B)
	var.names <- c(all.vars(formula), cat.var)
	X <- data[,cat.var]
	data.o <- data
	for (i in 1:B) {
		data.b <- bootstrap.sample(data, group = var.names[2],  method = b.method)
		sel.point = sort(unique(c(min(X, data.b[,cat.var]), max(X, data.b[,cat.var]), c.points)))
		x.cut.boot <- cut(data.b[,cat.var], sel.point, include.lowest = TRUE, right = TRUE)
		x.cut <-  cut(X, sel.point, include.lowest = TRUE, right = TRUE)
		if(length(levels(x.cut.boot)) == length(levels(x.cut)) & all(table(x.cut.boot)>1)) {
			data.b[,"x.cut_"] <- x.cut.boot
			data.o[,"x.cut_"] <- x.cut
			# Bootstrap
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
			f.boot <- rms::cph(formula.n, data=data.b)
			cind.boot[i] <- cindex.categorization(f.boot$linear.predictors, survival::Surv(data.b[,var.names[1]],data.b[,var.names[2]]))
			# Original Sample
			p <- stats::predict(f.boot, newdata = data.o, type = "lp")
			cind.original[i] <- cindex.categorization(p, survival::Surv(data.o[,var.names[1]],data.o[,var.names[2]]))
		} else {
			cind.boot[i] <- NA
			cind.original[i] <- NA
		}
	}
	cind.corrected <- cindex - mean(abs(cind.original - cind.boot), na.rm=TRUE)
	cind.corrected
}

select.cutpoint.auc <-
function(formula, cat.var, data, range, points, l.s.points = 100, min.p.cat = 1) {
	search.points = seq(min(range),max(range),l = l.s.points)
	auc.matrix = matrix(ncol = 2,nrow = length(search.points))
	colnames(auc.matrix)<- c("points","auc")
	var.names <- all.vars(formula)
	X <- data[,cat.var]
	Y <- data[,var.names[1]]
	for (i in 1:length(search.points)){
		cutoffs = sort(unique(c(min(X), max(X), points, search.points[i])))
		x.cut = cut(X, cutoffs, include.lowest = TRUE, right = TRUE)
		if(length(levels(x.cut)) > 1 & all(table(x.cut)>= min.p.cat)) {
			if(length(var.names) == 1) {
				ratio <- prop.table(table(x.cut[Y==1]))/prop.table(table(x.cut[Y==0]))
				ratio.s <- ratio[match(x.cut, names(ratio))]
				auc.matrix[i,2] <- compute.empirical.AUC(ratio.s[Y==1], ratio.s[Y==0])
			} else {
				data[,"x.cut_"] <- x.cut
				formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
				fit <- mgcv::gam(formula.n, family = stats::binomial, data = data)
				auc.matrix[i,2] <- compute.empirical.AUC(fit$fitted[Y==1], fit$fitted[Y==0])
			}
			auc.matrix[i,1] <- search.points[i]
		 } else {
			auc.matrix[i,2] <- NA
		 }
	}
	auc.matrix
}

#' Function to obtain optimal cut points to categorise a continuous predictor
#'    variable in a logistic regression model
#'
#' @param formula An object of class \code{\link{formula}} giving the model to be
#'    fitted in addition to the continuous covariate is aimed to categorise.
#'    This argument allows the user to specify whether the continuous predictor
#'    should be categorised in a univariable context, or in presence of other
#'    covariates or cofounders, i.e in a multiple logistic regression model.
#'    For instance, \code{Y ~ 1} indicates that the categorisation should be done in a
#'    univariable setting, with Y being the response variable. If the predictor
#'    variable is aimed to be categorised in a multivariable setting, this argument
#'    allows to specify whether the covariates should be modelled using linear
#'    or non linear effects. In the latest, the effects are estimated using the
#'    \pkg{mgcv} package.
#' @param cat.var Name of the continuous variable to categorise.
#' @param cat.points Number of cut points to look for.
#' @param data Data frame containing all needed variables.
#' @param method The algorithm selected to search for the optimal cut points.
#'    \code{"addfor"} if the AddFor algorithm is choosen, \code{"backaddfor"} if
#'    the BackAddFor algorithm is selected and \code{"genetic"} otherwise.
#' @param range The range of the continuous variable in which to look for the cut
#'    points. By default \code{NULL}, i.e, all the range.
#' @param correct.AUC A logical value. If \code{TRUE} the bias corrected AUC is estimated.
#' @param control Output of the \code{\link{controlcatpredi}} function.
#' @param ... Further arguments for passing on to the function \code{\link[rgenoud]{genoud}}
#'    of the package \pkg{rgenoud}.
#'
#' @return
#' Returns an object of class \code{"catpredi"} with the following components:
#' \describe{
#'   \item{call}{The matched call.}
#'   \item{method}{The algorithm selected in the call.}
#'   \item{formula}{The model formula used in the call.}
#'   \item{cat.var}{Name of the continuous variable to categorise.}
#'   \item{data}{The data frame used in the call.}
#'   \item{correct.AUC}{Logical value indicating whether bias-corrected AUC was used.}
#'   \item{results}{A list containing estimated cut points, AUC and
#'     bias-corrected AUC for each method.}
#'   \item{control}{The control parameters used in the call.}
#' }
#'
#' @description Returns an object with the optimal cut points to categorise a
#'  continuous predictor variable in a logistic regression model
#'
#' @references I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number
#'    of categories of the lymph node ratio in cancer research: A bootstrap-based
#'    hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
#'
#'    I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
#'    A new approach to categorising continuous variables in prediction models:
#'     proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.
#'
#'    S.N Wood (2006). Generalized Additive Models: An Introduction with R. Chapman and Hall/CRC.
#'
#' @author Irantzu Barrio, Maria Xose Rodriguez-Alvarez, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
#'
#' @seealso
#' \code{\link{controlcatpredi}},
#' \code{\link{comp.cutpoints}},
#' \code{\link{plot.catpredi}},
#' \code{\link{summary.catpredi}}
#'
#' @examples
#' library(CatPredi)
#' \dontrun{
#' set.seed(127)
#' #Simulate data
#' n = 100
#' #Predictor variable
#' xh <- rnorm(n, mean = 0, sd = 1)
#' xd <- rnorm(n, mean = 1.5, sd = 1)
#' x <- c(xh, xd)
#' #Response
#' y <- c(rep(0,n), rep(1,n))
#' #Covariate
#' zh <- rnorm(n, mean=1.5, sd=1)
#' zd <- rnorm(n, mean=1, sd=1)
#' z <- c(zh, zd)
#' # Data frame
#' df <- data.frame(y = y, x = x, z = z)
#'
#' # Select optimal cut points using the AddFor algorithm
#' res.addfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 2,
#'                        data = df, method = "addfor", range=NULL, correct.AUC=FALSE,
#'                        control=controlcatpredi(grid=20))
#'
#' # Select optimal cut points using the BackAddFor algorithm
#' res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
#'                            data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
#' }
#' \dontrun{
#'   set.seed(127)
#'   #Simulate data
#'   n = 200
#'   #Predictor variable
#'   xh <- rnorm(n, mean = 0, sd = 1)
#'   xd <- rnorm(n, mean = 1.5, sd = 1)
#'   x <- c(xh, xd)
#'   #Response
#'   y <- c(rep(0,n), rep(1,n))
#'   #Covariate
#'   zh <- rnorm(n, mean=1.5, sd=1)
#'   zd <- rnorm(n, mean=1, sd=1)
#'   z <- c(zh, zd)
#'   # Data frame
#'   df <- data.frame(y = y, x = x, z = z)
#'
#'   # Select optimal cut points using the AddFor algorithm
#'   res.addfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
#'                          data = df, method = "addfor", range=NULL, correct.AUC=FALSE)
#'
#'   # Select optimal cut points using the BackAddFor algorithm
#'   res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
#'                              data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
#' }
#'
#' @export
catpredi <-
function(formula, cat.var, cat.points = 1, data, method = c("addfor","genetic","backaddfor"), range = NULL, correct.AUC = FALSE, control = controlcatpredi(), ...) {
	control <- do.call("controlcatpredi", control)

	if(missing(formula)) {
		stop("Argument \"formula\" is missing, with no default")
	}
	if(missing(data)) {
		stop("Argument \"data\" is missing, with no default")
	}
	if(missing(cat.var)) {
		stop("Argument \"cat.var\" is missing, with no default")
	}
	var.names <- c(all.vars(formula), cat.var)
	if(!all(var.names %in% names(data))) {
		stop("Not all needed variables are supplied in \"data\"")
	}
	data.res <- stats::na.omit(data[,var.names])
	unique.resp <- unique(data.res[,var.names[1]])
	if(length(unique.resp) != 2 || !is.numeric(unique.resp) || !all(unique.resp %in% c(0,1))) {
		stop("The response variable should be numeric and codified as 0 (healthy) and 1 (diseased)")
	}
	method <- match.arg(method)
	if(is.null(range)) {
		range <- range(data.res[,cat.var])
	}
	# Call the methods
	if(method == "addfor") {
		res <- k.points.max.auc(formula = formula, cat.var = cat.var, data = data.res,
		                        range = range, k = cat.points, l.s.points = control$grid,
		                        min.p.cat = control$min.p.cat)
		cutpoints <- res[,1]
		AUC = res[,2]

		if(correct.AUC == TRUE) {
			AUC.cor <- auc.opt.corrected(formula = formula, cat.var = cat.var, data = data.res ,
			                             c.points = cutpoints, AUC = AUC[length(cutpoints)], B=control$B ,
			                             b.method = control$b.method)
	 	} else {
			AUC.cor <- NULL
	 	}
	} else if(method == "genetic"){
		Dim <- matrix(ncol = 2, nrow = cat.points)
		Dim[,1] = range[1]*1.0
		Dim[,2] = range[2]*1.0
		res <- rgenoud::genoud(calculate.AUC, cat.points, max = TRUE, formula = formula,
		                       cat.var = cat.var, data.f = data.res, range = range,
		                       min.p.cat = control$min.p.cat, Domains = Dim, print.level = control$print.gen, ...)
		cutpoints <- res$par
		AUC = res$value

		if(correct.AUC == TRUE) {
			AUC.cor <- auc.opt.corrected(formula = formula, cat.var = cat.var, data = data.res ,
			                             c.points = cutpoints, AUC = AUC, B=control$B , b.method = control$b.method)
		} else {
			AUC.cor <- NULL
		}
	} else { # method == "backaddfor"
	  res <- backaddfor(formula = formula, cat.var = cat.var, data = data.res, range = range,
	                    k = cat.points, l.s.points = control$grid, min.p.cat = control$min.p.cat,
	                    eps = control$eps, repmax = control$B, ...)
	  cutpoints <- res$cuts
	  AUC = res$auc

	  if(correct.AUC == TRUE) {
	    AUC.cor <- auc.opt.corrected(formula = formula, cat.var = cat.var, data = data.res ,
	                                 c.points = cutpoints, AUC = AUC, B=control$B , b.method = control$b.method)
	  } else {
	    AUC.cor <- NULL
	  }
	}
	# Create the categorical covariate
	data[,paste0(cat.var,"_CatPredi")] <- cut(data[,cat.var], sort(unique(c(max(data[,cat.var], na.rm=TRUE),
	                                                                        min(data[,cat.var], na.rm=TRUE),
	                                                                        cutpoints))), include.lowest = TRUE, right = TRUE)
	results <- if(method == "addfor" & correct.AUC == TRUE) {
				list(cutpoints = cutpoints, AUC = AUC, AUC.cor = AUC.cor,  grid = control$grid)
			} else if(method == "genetic" & correct.AUC == TRUE) {
				list(cutpoints = cutpoints, AUC = AUC, AUC.cor = AUC.cor)
			} else if(method == "backaddfor" & correct.AUC == TRUE){
			  list(cutpoints = cutpoints, AUC = AUC, AUC.cor = AUC.cor,  grid = control$grid )
			} else if(method == "addfor" & correct.AUC == FALSE) {
				list(cutpoints = cutpoints, AUC = AUC, grid = control$grid)
			} else if(method == "genetic" & correct.AUC == FALSE){
			  list(cutpoints = cutpoints, AUC = AUC)
			} else{ # method == "backaddfor" & correct.AUC == FALSE
			  list(cutpoints = cutpoints, AUC = AUC, grid = control$grid)
			}

	res <- list(call = match.call(), method = method, formula = formula, cat.var = cat.var, data = data,
	            correct.AUC = correct.AUC, results = results , control = control)
	class(res) <- "catpredi"
	res
}

#' Plot the optimal cut points.
#'
#' @param x An object of type catpredi.survival.
#' @param ... Additional arguments to be passed on to other functions. Not yet implemented.
#'
#' @returns
#' This function returns the plot of the relationship between the predictor variable and the outcome.
#'
#' @description Plots the functional form of the predictor variable we want to
#'    categorise. Additionally, the optimal cut points obtained with the
#'    catpredi.survival() function are drawn on the graph.
#'
#' @references I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
#'    I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
#'     of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92
#'
#' @author Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
#'
#' @seealso
#' \code{\link{catpredi.survival}}
#'
#' @examples
#' library(CatPredi)
#' library(survival)
#' set.seed(123)
#' #Simulate data
#' n = 500
#' tauc = 1
#' X <- rnorm(n=n, mean=0, sd=2)
#' SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
#' # Censoring time
#' CensTime <- runif(n=n, min=0, max=tauc)
#' # Status
#' SurvS <- as.numeric(SurvT <= CensTime)
#' # Data frame
#' dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)
#'
#' # Select optimal cut points using the AddFor algorithm
#' res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
#'                           data = dat, method = "addfor", conc.index = "cindex", range = NULL,
#'                           correct.index = FALSE)
#' # Plot
#' plot(res)
#'
#' @export
plot.catpredi.survival <-
function(x, ...){
	# Fit the model
	formula <- stats::update(x$formula, stats::as.formula(paste("~ . + pspline(", x$cat.var, ")", sep = "")))
	formula <- stats::as.formula(Reduce(paste, deparse(formula)))
	fit.survival <- survival::coxph(formula, data = x$data)
	pos <- attr(stats::terms.formula(formula, specials = c("pspline")),"specials")$pspline - 1
	stats::termplot(fit.survival, terms = pos, se = TRUE, ylabs = paste0("f(",x$cat.var,")"))
	graphics::abline(v = x$results$cutpoints, lty = 2)
}

#' Summary method for catpredi objects
#'
#' @param object An object of class catpredi as produced by catpredi()
#' @param digits .
#' @param ... Further arguments passed to or from other methods.
#'
#' @returns
#' Returns an object of class "summary.catpredi" with the same components as the
#'    catpredi function (see \code{\link{catpredi}}). plus:
#' \describe{
#'   \item{fit.gam}{fitted model according to the model specified in the call,
#'     based on the function \code{\link[mgcv]{gam}} of the package \pkg{mgcv}.}
#' }
#'
#' @description Produces a summary of a catpredi object. The following are printed:
#'    the call to the catpredi() function; the estimated optimal cut points obtained
#'    with the method selected and the estimated AUC and bias corrected AUC (if the
#'    argument correct.AUC is TRUE) for the categorised variable.
#'
#' @references I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
#'    A new approach to categorising continuous variables in prediction models:
#'    proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.
#'
#'    I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number of
#'    categories of the lymph node ratio in cancer research: A bootstrap-based
#'    hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
#'
#' @author Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
#'
#' @seealso
#' \code{\link{catpredi}}
#'
#' @examples
#' library(CatPredi)
#' set.seed(127)
#' #Simulate data
#' n = 200
#' #Predictor variable
#' xh <- rnorm(n, mean = 0, sd = 1)
#' xd <- rnorm(n, mean = 1.5, sd = 1)
#' x <- c(xh, xd)
#' #Response
#' y <- c(rep(0,n), rep(1,n))
#' #Covariate
#' zh <- rnorm(n, mean=1.5, sd=1)
#' zd <- rnorm(n, mean=1, sd=1)
#' z <- c(zh, zd)
#' # Data frame
#' df <- data.frame(y = y, x = x, z = z)
#'
#' # Select optimal cut points using the AddFor algorithm
#' res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 2,
#'                            data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
#' # Summary
#' summary(res.backaddfor)
#'
#' @export
summary.catpredi <-
function(object, digits = 4, ...) {
	object$digits <- digits
	var.names <- all.vars(object$formula)
  formula <- object$formula
  data <- object$data
  X <- data[,object$cat.var]
  Y <- data[,var.names[1]]
  cutoffs <- sort(unique(c(max(X, na.rm=TRUE), min(X, na.rm=TRUE), object$results$cutpoints)))
  x.cut <- cut(X, cutoffs, include.lowest=TRUE,right=TRUE)
  name_var_cat <- paste(object$cat.var,"_cat",sep="")
  data[,name_var_cat] <- x.cut
  new.formula <- paste("~ . + ", name_var_cat, sep = "")
  formula.n <- stats::update(formula, stats::as.formula(new.formula))
  fit <- mgcv::gam(formula.n, family = stats::binomial, data = data)
  object$fit.gam <- fit
	class(object) <- "summary.catpredi"
  return(object)
}

#' Selection of optimal number of cut points
#'
#' @param obj1 An object inheriting from class \code{"catpredi.survival"} for k number of cut points
#' @param obj2 An object inheriting from class \code{"catpredi.survival"} for k+1 number of cut points
#' @param V Number of bootstrap resamples. By default V=100
#'
#' @returns
#' This function returns an object of class "comp.cutpoints.survival" with the following components:
#' \describe{
#'   \item{CI.cor.diff}{the difference of the bias corrected concordance probability for the two categorical variables.}
#'   \item{icb.CI.diff}{bootstrap based confidence interval for the bias corrected concordance probability difference.}
#' }
#'
#' @description Compares two objects of class \code{"catpredi.survival"}
#'
#' @references I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and  I Arostegui (2017).
#'    Comparison of two discrimination indexes in the categorisation of continuous predictors in time-to-event studies.
#'    \emph{SORT}, 41:73-92
#'
#' @author Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
#'
#' @seealso
#' \code{\link{catpredi.survival}}
#'
#' @examples
#' library(CatPredi)
#' library(survival)
#' set.seed(123)
#'
#' #Simulate data
#' n = 300
#' tauc = 1
#' X <- rnorm(n=n, mean=0, sd=2)
#' SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
#' # Censoring time
#' CensTime <- runif(n=n, min=0, max=tauc)
#' # Status
#' SurvS <- as.numeric(SurvT <= CensTime)
#' # Data frame
#' dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)
#' \donttest{
#'   # Select 2 optimal cut points using the AddFor algorithm. Correct the c-index
#'     res.k2 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
#'                                  data = dat, method = "addfor", conc.index = "cindex",
#'                                  range = NULL, correct.index = TRUE)
#'   # Select 3 optimal cut points using the AddFor algorithm. Correct the c-index
#'     res.k3 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 3,
#'                                  data = dat, method = "addfor", conc.index = "cindex",
#'                                  range = NULL, correct.index = TRUE)
#'   # Select optimal number of cut points
#'     comp <-  comp.cutpoints.survival(res.k2, res.k3, V = 100)
#' }
#' \dontshow{
#'   # Select 2 optimal cut points using the AddFor algorithm. Correct the c-index
#'     res.k2 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 1,
#'                                  data = dat, method = "addfor", conc.index = "cindex",
#'                                  range = NULL, correct.index = TRUE,
#'                                  control=controlcatpredi.survival(grid=20))
#'   # Select 3 optimal cut points using the AddFor algorithm. Correct the c-index
#'     res.k3 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
#'                                  data = dat, method = "addfor", conc.index = "cindex",
#'                                  range = NULL, correct.index = TRUE,
#'                                  control=controlcatpredi.survival(grid=20))
#'   # Select optimal number of cut points
#'     comp <-  comp.cutpoints.survival(res.k2, res.k3, V = 2)
#' }
#'
#' @export
comp.cutpoints.survival <-
function(obj1,obj2, V=100) {
	if(obj1$correct.index==FALSE || obj2$correct.index==FALSE ) {
			stop("argument correct.index=TRUE is needed in catpredi.survival")
	}
	if(obj1$formula !=obj2$formula) {
		stop("The categorized variables are not comparable")
	}
	if(obj1$conc.index != obj2$conc.index) {
		stop("The concordance index selected in both objects must be the same")
	}
	if(obj1$control$B != obj2$control$B) {
		warning("The bootstrap resamples used for the optimism correction is different in both objects")
	}
	formula <- obj1$formula
	point1 <- obj1$results$cutpoints
	point2 <- obj2$results$cutpoints
	B1 <-   obj1$control$B
  B2 <-   obj2$control$B
  b.method <- obj1$control$b.method
	data <-   obj1$data
	cat.var <- obj1$cat.var
	var.names <- c(all.vars(formula), cat.var)
	X <- data[,cat.var]
	ci.b.1 <- ci.b.2 <- ci.b.diff <- vector(length = V)

	if(obj1$conc.index=="cindex") {
		ci.cor.diff <- obj2$results$Cindex.cor - obj1$results$Cindex.cor
		for (i in 1:V) {
			data.b <- bootstrap.sample(data, group = var.names[2], method = b.method)
			X.b <- data.b[,cat.var]
			# k
			sel.point = sort(unique(c(min(X,X.b), max(X,X.b), point1)))
			x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
			data.b[,"x.cut1"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut1"))

			fit.1 <- rms::cph(formula.n, data=data.b)
			cind.fit1 <- cindex.categorization(fit.1$linear.predictors, survival::Surv(data.b[,var.names[1]],data.b[,var.names[2]]))
			ci.b.1[i] <- cindex.opt.corrected(formula = formula, cat.var =  cat.var , data = data.b , c.points = point1 , cindex = cind.fit1 , B=B1 , b.method = b.method)

			# k = k+1
			sel.point = sort(unique(c(min(X,X.b), max(X,X.b), point2)))
			x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
			data.b[,"x.cut2"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut2"))

			fit.2 <- rms::cph(formula.n, data=data.b)
			cind.fit2 <- cindex.categorization(fit.2$linear.predictors, survival::Surv(data.b[,var.names[1]],data.b[,var.names[2]]))
			ci.b.2[i] <- cindex.opt.corrected(formula = formula, cat.var =  cat.var , data = data.b , c.points = point2 , cindex = cind.fit2 , B=B2 , b.method = b.method)

			ci.b.diff[i] <- ci.b.2[i] - ci.b.1[i]
		}
	} else {
		ci.cor.diff <- obj2$results$CPE.cor - obj1$results$CPE.cor
		for (i in 1:V) {
			data.b <- bootstrap.sample(data,var.names[2], b.method)
			X.b <- data.b[,cat.var]
			# k
			sel.point = sort(unique(c(min(X,X.b , na.rm=TRUE), max(X,X.b , na.rm=TRUE), point1)))
			x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
			data.b[,"x.cut1"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut1"))
			fit.1 <- rms::cph(formula.n, data=data.b)
			cpe.fit1 <- CPE::phcpe2(coef = fit.1$coefficients, coef.var = fit.1$var, design = stats::model.matrix(fit.1, data = data.b))$CPE #phcpe(fit.1,CPE.SE=FALSE, out.ties=FALSE)$CPE
			ci.b.1[i] <- cpe.opt.corrected(formula = formula, cat.var =  cat.var , data = data.b , c.points = point1 , cpe = cpe.fit1 , B = B1 , b.method = b.method)

			# k = k+1
			sel.point = sort(unique(c(min(X,X.b , na.rm=TRUE), max(X,X.b , na.rm=TRUE), point2)))
			x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
			data.b[,"x.cut2"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut2"))

			fit.2 <- rms::cph(formula.n, data=data.b)
			# cpe.fit2 <- coxcpe(fit.2, data.b)
			cpe.fit2 <- CPE::phcpe2(coef = fit.2$coefficients, coef.var = fit.2$var, design = stats::model.matrix(fit.2, data = data.b))$CPE #phcpe(fit.2,CPE.SE=FALSE, out.ties=FALSE)$CPE
			ci.b.2[i] <- cpe.opt.corrected(formula = formula, cat.var =  cat.var , data = data.b , c.points = point2 , cpe = cpe.fit2 , B = B2, b.method = b.method)

			ci.b.diff[i] <- ci.b.2[i] - ci.b.1[i]
		}
	}
	ci.diff <- stats::quantile(ci.b.diff, p = c(0.025, 0.975), na.rm=TRUE)
	res <- list( call = match.call() , CI.cor.diff = ci.cor.diff , icb.CI.diff = ci.diff)
	class(res) <- "comp.cutpoints.survival"
	res
}

#' Summary method for objects of class "catpredi.survival"
#'
#' @param object An object of class "catpredi.survival" as produced by catpredi.survival()
#' @param digits .
#' @param ... Further arguments passed to or from other methods.
#'
#' @returns
#' Returns an object of class "summary.catpredi.survival" with the same components
#'    as the catpredi.survival function (see \code{\link{catpredi.survival}}).
#'
#' @description Produces a summary of a "catpredi.survival" object. The following
#'    are printed: the call to the catpredi.survival() function; the estimated
#'    optimal cut points obtained with the method and concordance probability
#'    estimator selected and the estimated and bias corrected concordance probability
#'    for the categorised variable (whenever the argument correct.index is set to TRUE).
#'
#' @references I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
#'    I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
#'    of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92
#'
#' @author Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
#'
#' @seealso
#' \code{\link{catpredi.survival}}
#'
#' @examples
#' library(CatPredi)
#' library(survival)
#' set.seed(123)
#' #Simulate data
#' n = 500
#' tauc = 1
#' X <- rnorm(n=n, mean=0, sd=2)
#' SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
#' # Censoring time
#' CensTime <- runif(n=n, min=0, max=tauc)
#' # Status
#' SurvS <- as.numeric(SurvT <= CensTime)
#' # Data frame
#' dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)
#'
#' # Select optimal cut points using the AddFor algorithm
#' res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
#'                           data = dat, method = "addfor", conc.index = "cindex", range = NULL,
#'                           correct.index = FALSE)
#' # Summary
#' summary(res)
#'
#' @export
summary.catpredi.survival <-
function(object, digits = 4, ...) {
	object$digits <- digits
	class(object) <- "summary.catpredi.survival"
	object
}

#' @export
print.summary.catpredi <-
function(x, ...) {
  print.catpredi(x, digits = x$digits)

  cat("\n\n---------------------------------------------------\n")
  cat("Fitted model for the categorized predictor variable\n")
  cat("---------------------------------------------------\n\n")
  tpm <- summary(x$fit.gam)
  digits = max(3, getOption("digits") - 3)
  signif.stars = getOption("show.signif.stars")
   print(tpm$family)
      cat("Formula:\n")
      if (is.list(tpm$formula))
          for (i in 1:length(tpm$formula)) print(tpm$formula[[i]])
      else print(tpm$formula)
      if (length(tpm$p.coeff) > 0) {
          cat("\nParametric coefficients:\n")
          stats::printCoefmat(tpm$p.table, digits = digits, signif.stars = signif.stars,
              na.print = "NA", ...)
      }
      cat("\n")
      if (tpm$m > 0) {
          cat("Approximate significance of smooth terms:\n")
          stats::printCoefmat(tpm$s.table, digits = digits, signif.stars = signif.stars,
              has.Pvalue = TRUE, na.print = "NA", cs.ind = 1, ...)
      }
      cat("\n")
      if (!is.null(tpm$rank) && tpm$rank < tpm$np)
          cat("Rank: ", tpm$rank, "/", tpm$np, "\n", sep = "")

      invisible(tpm)

}

coxcpe <- function(phfit, data) {
  xbeta <- stats::na.omit(stats::predict(phfit, data, type = "lp"))
  n <- length(xbeta)
  CPE <- 0
  for (i in c(1:(n-1))) {
    for (j in (i+1):n) {
      xbij <- xbeta[i] - xbeta[j]
      eij <- 1 + exp(xbij)
      eji <- 1 + exp(-xbij)
      if (xbij<= 0){
        CPE <-  CPE + 1/eij
      }else{
        CPE <-  CPE + 1/eji
      }
    }
  }
  CPE <- 2*CPE/(n*(n-1))
  as.numeric(CPE)
}

#' Function to obtain optimal cut points to categorise a continuous predictor variable
#'    in a Cox proportional hazards regression model
#'
#' @param formula An object of class \code{\link{formula}} giving the model to be
#'    fitted in addition to the continuous covariate is aimed to categorise.
#'    The response must be a survival object as returned by the \code{\link[survival]{Surv}}
#'    function. This argument allows the user to specify whether the continuous
#'    predictor should be categorised in a univariable context, or in presence of
#'    other covariates or cofounders, i.e in a multiple Cox proportional hazards
#'    regression model. For instance, Surv(SurvT,SurvS)~1 indicates that the
#'    categorisation should be done in a univariable setting.
#' @param cat.var Name of the continuous variable to categorise.
#' @param cat.points Number of cut points to look for.
#' @param data Data frame containing all needed variables.
#' @param method The algorithm selected to search for the optimal cut points.
#'    \code{"addfor"} if the AddFor algorithm is choosen, \code{"backaddfor"} if
#'    the BackAddFor algorithm is selected and \code{"genetic"} otherwise.
#' @param conc.index The concordance probability estimator selected for maximisation
#'    purposes. "cindex" if the c-index concordance probability is choosen and
#'    "cpe" otherwise. The c-index and CPE are estimated using the \pkg{rms} and
#'    \pkg{CPE} packages, respectively.
#' @param range The range of the continuous variable in which to look for the cut
#'    points. By default \code{NULL}, i.e, all the range.
#' @param correct.index A logical value. If TRUE the bias corrected concordance
#'    probability is estimated.
#' @param control Output of the \code{\link{controlcatpredi.survival}} function.
#' @param ... Further arguments for passing on to the function \code{\link[rgenoud]{genoud}}
#'    of the package \pkg{rgenoud}.
#'
#' @returns
#' Returns an object of class \code{"catpredi.survival"} with the following components:
#' \describe{
#'   \item{call}{The matched call.}
#'   \item{method}{The algorithm selected in the call.}
#'   \item{formula}{an object of class \code{\link{formula}} giving the model to
#'      be fitted in addition to the continuous covariate is aimed to categorise.}
#'   \item{cat.var}{name of the continuous variable to categorise.}
#'   \item{data}{the data frame with the variables used in the call.}
#'   \item{correct.index}{The logical value used in the call.}
#'   \item{results}{a list with the estimated cut points, concordance probability
#'       and bias corrected concordance probability.}
#'   \item{control}{the control parameters used in the call.}
#'   When the c-index concordance probability is choosen, a list with the following
#'   components is obtained for each of the methods used in the call:
#'   \item{"cutpoints"}{Estimated optimal cut points.}
#'   \item{"Cindex"}{Estimated c-index.}
#'   \item{"Cindex.cor"}{Estimated bias corrected c-index.}
#'   When the CPE concordance probability is choosen, a list with the following
#'   components is obtained for each of the methods used in the call:
#'   \item{"cutpoints"}{Estimated optimal cut points.}
#'   \item{"CPE"}{Estimated CPE.}
#'   \item{"CPE.cor"}{Estimated bias corrected CPE.}
#' }
#'
#' @description Returns an object with the optimal cut points to categorise a continuous
#'    predictor variable in a Cox proportional hazards regression model
#'
#' @references I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
#'    I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
#'    of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92
#'
#'    M Gonen and  G Heller (2005). Concordance probability and discriminatory power
#'    in  proportional hazards regression. \emph{Biometrika}, 92:965-970.
#'
#'    F Harrell (2001). Regression modeling strategies: with applications to linear models,
#'    logistic and ordinal regression, and survival analysis. Springer.
#'
#' @author Irantzu Barrio and Maria Xose Rodriguez-Alvarez
#'
#' @seealso
#' \code{\link{controlcatpredi.survival}},
#' \code{\link{comp.cutpoints.survival}},
#' \code{\link{plot.catpredi.survival}},
#' \code{\link{catpredi}}
#'
#' @examples
#' library(CatPredi)
#' library(survival)
#' set.seed(123)
#' #Simulate data
#' n = 500
#' tauc = 1
#' X <- rnorm(n=n, mean=0, sd=2)
#' SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
#' # Censoring time
#' CensTime <- runif(n=n, min=0, max=tauc)
#' # Status
#' SurvS <- as.numeric(SurvT <= CensTime)
#' # Data frame
#' dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)
#'
#' # Select optimal cut points using the AddFor algorithm
#' res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
#'                           data = dat, method = "addfor", conc.index = "cindex", range = NULL,
#'                           correct.index = FALSE)
#'
#' @export
catpredi.survival <-
function(formula, cat.var, cat.points = 1, data, method = c("addfor","genetic","backaddfor"), conc.index = c("cindex","cpe"), range = NULL, correct.index = FALSE, control = controlcatpredi.survival(), ...) {
	control <- do.call("controlcatpredi.survival", control)

	if(missing(formula)) {
		stop("Argument \"formula\" is missing, with no default")
	}
	if(is.character(formula))
		formula = stats::as.formula(formula)
	if(missing(data)) {
		stop("Argument \"data\" is missing, with no default")
	}
	if(missing(cat.var)) {
		stop("Argument \"cat.var\" is missing, with no default")
	}
	var.names <- c(all.vars(formula), cat.var)
	if(!all(var.names %in% names(data))) {
		stop("Not all needed variables are supplied in \"data\"")
	}
	data.res <- stats::na.omit(data[,var.names])
	unique.resp <- unique(data.res[,var.names[2]])
	if(length(unique.resp) != 2 || !is.numeric(unique.resp) || !all(unique.resp %in% c(0,1))) {
		stop("The event status indicator should be numeric and codified as 0 (censored) and 1 (event at time)")
	}
	## COMO PODEMOS ASEGURAR QUE SEA UN OBJETO DE TIPO SURV??
	method <- match.arg(method)
	conc.index <- match.arg(conc.index)

	if(is.null(range)) {
		range <- range(data.res[,cat.var])
	}
	# Call the methods
	if(method == "addfor" & conc.index == "cindex") {
		res <- k.points.max.cind(formula = formula, cat.var = cat.var, data = data.res, range = range, k = cat.points, l.s.points = control$grid, min.p.cat = control$min.p.cat)
		cutpoints <- res[,1]
		Cindex = res[,2]
		# Correct the C-index
		if(correct.index == TRUE) {
			Cindex.cor <- cindex.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cindex = Cindex[length(cutpoints)] , B=control$B, b.method = control$b.method )
	 	} else {
			Cindex.cor <- NULL
		}
	} else if(method == "addfor" & conc.index == "cpe") {
		res <- k.points.max.cpe(formula = formula, cat.var = cat.var, data = data.res, range = range, k = cat.points, l.s.points = control$grid, min.p.cat = control$min.p.cat)
		cutpoints <- res[,1]
		CPE = res[,2]
		# Correct the CPE
		if(correct.index == TRUE) {
			CPE.cor <- cpe.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cpe = CPE[length(cutpoints)] , B=control$B , b.method = control$b.method)
		} else {
			CPE.cor <- NULL
		}
	} else if(method == "genetic" & conc.index == "cindex") {
		Dim <- matrix(ncol = 2, nrow = cat.points)
		Dim[,1] = range[1]*1.0
		Dim[,2] = range[2]*1.0
		res <- rgenoud::genoud(calculate.cind, cat.points, max = TRUE, formula = formula, cat.var = cat.var, data.f = data.res, range = range, min.p.cat = control$min.p.cat, Domains = Dim, print.level = control$print.gen, ...)
		cutpoints <- res$par
		Cindex = res$value
		# Correct the C-index
		if(correct.index == TRUE) {
			Cindex.cor <- cindex.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cindex = Cindex , B=control$B, b.method = control$b.method)
		} else {
			Cindex.cor <- NULL
		}
	} else if(method == "genetic" & conc.index == "cpe"){
		Dim <- matrix(ncol = 2, nrow = cat.points)
		Dim[,1] = range[1]*1.0
		Dim[,2] = range[2]*1.0
		res <- rgenoud::genoud(calculate.CPE, cat.points, max = TRUE, formula = formula, cat.var = cat.var, data.f = data.res, range = range, min.p.cat = control$min.p.cat, Domains = Dim, print.level = control$print.gen, ...)
		cutpoints <- res$par
		CPE = res$value
		# Correct the CPE
		if(correct.index == TRUE) {
			CPE.cor <- cpe.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cpe = CPE , B=control$B , b.method = control$b.method)
		} else {
			CPE.cor <- NULL
		}
	}  else if(method == "backaddfor" & conc.index == "cindex") {
	  Dim <- matrix(ncol = 2, nrow = cat.points)
	  Dim[,1] = range[1]*1.0
	  Dim[,2] = range[2]*1.0
	  res <- backaddfor.cind(formula = formula, cat.var = cat.var, data = data.res, range = range, k = cat.points, l.s.points = control$grid, min.p.cat = control$min.p.cat, eps = control$eps, repmax = control$B)
	  # res <- rgenoud::genoud(calculate.cind, cat.points, max = TRUE, formula = formula, cat.var = cat.var, data.f = data.res, range = range, min.p.cat = control$min.p.cat, Domains = Dim, print.level = control$print.gen, ...)
	  cutpoints <- res$cuts
	  Cindex = res$cind
	  # Correct the C-index
	  if(correct.index == TRUE) {
	    Cindex.cor <- cindex.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cindex = Cindex , B=control$B, b.method = control$b.method)
	  } else {
	    Cindex.cor <- NULL
	  }
	} else { #if(method == "backaddfor" & conc.index == "cpe"){
	  Dim <- matrix(ncol = 2, nrow = cat.points)
	  Dim[,1] = range[1]*1.0
	  Dim[,2] = range[2]*1.0
	  res <- backaddfor(formula = formula, cat.var = cat.var, data = data.res, range = range, k = cat.points, l.s.points = control$grid, min.p.cat = control$min.p.cat, eps = control$eps, repmax = control$B, ...)
	  # res <- rgenoud::genoud(calculate.CPE, cat.points, max = TRUE, formula = formula, cat.var = cat.var, data.f = data.res, range = range, min.p.cat = control$min.p.cat, Domains = Dim, print.level = control$print.gen, ...)
	  cutpoints <- res$cuts
	  CPE = res$auc
	  # Correct the CPE
	  if(correct.index == TRUE) {
	    CPE.cor <- cpe.opt.corrected(formula = formula, cat.var =  cat.var , data = data.res , c.points = cutpoints , cpe = CPE , B=control$B , b.method = control$b.method)
	  } else {
	    CPE.cor <- NULL
	  }
	}
	# Create the categorical covariate
	data[,paste0(cat.var,"_CatPredi")] <- cut(data[,cat.var], sort(unique(c(max(data[,cat.var]), min(data[,cat.var]), cutpoints))), include.lowest = TRUE, right = TRUE)
	results <- if(method == "addfor" & conc.index == "cindex" & correct.index == TRUE ) {
					list(cutpoints = cutpoints, Cindex = Cindex, Cindex.cor = Cindex.cor,  grid = control$grid)
				} else if(method == "addfor" & conc.index == "cpe" & correct.index == TRUE) {
					list(cutpoints = cutpoints, CPE = CPE, CPE.cor = CPE.cor,  grid = control$grid)
				} else if(method == "genetic" & conc.index == "cindex" & correct.index == TRUE) {
					list(cutpoints = cutpoints, Cindex = Cindex, Cindex.cor = Cindex.cor)
				} else if(method == "genetic" & conc.index == "cpe" & correct.index == TRUE) {
					list(cutpoints = cutpoints, CPE = CPE, CPE.cor = CPE.cor)
				} else if(method == "backaddfor" & conc.index == "cindex" & correct.index == TRUE) {
				  list(cutpoints = cutpoints, Cindex = Cindex, Cindex.cor = Cindex.cor)
				} else if(method == "backaddfor" & conc.index == "cpe" & correct.index == TRUE) {
				  list(cutpoints = cutpoints, CPE = CPE, CPE.cor = CPE.cor)
				}else if(method == "addfor" & conc.index == "cindex" & correct.index == FALSE ) {
					list(cutpoints = cutpoints, Cindex = Cindex, grid = control$grid)
				} else if(method == "addfor" & conc.index == "cpe" & correct.index == FALSE) {
					list(cutpoints = cutpoints, CPE = CPE, grid = control$grid)
				} else if(method == "genetic" & conc.index == "cindex" & correct.index == FALSE) {
					list(cutpoints = cutpoints, Cindex = Cindex)
				} else if (method == "genetic" & conc.index == "cpe" & correct.index == FALSE){
					list(cutpoints = cutpoints, CPE = CPE)
				} else if(method == "backaddfor" & conc.index == "cindex" & correct.index == FALSE) {
				  list(cutpoints = cutpoints, Cindex = Cindex)
				} else { #if (method == "backaddfor" & conc.index == "cpe" & correct.index == FALSE){
				  list(cutpoints = cutpoints, CPE = CPE)
				}
	res <- list(call = match.call(), method = method, conc.index = conc.index, formula = formula, cat.var = cat.var, data = data, correct.index = correct.index, results = results, control =  control)
	class(res) <- "catpredi.survival"
	res
}

backaddfor <-
function(formula, cat.var, data, range = NULL, k = 1, l.s.points = 100, eps = 0.001, repmax = 50, min.p.cat = 1, randnodes = FALSE){

  Y <- data[,all.vars(formula)[1]]
  if(is.null(range)) {
    X <- data[[cat.var]]
  } else{
    X <- data[[cat.var]][data[[cat.var]] >= range[1] & data[[cat.var]] <= range[2]]
  }
  # range is a vector such as: range=c(a,b)

  # Initial nodes
  cuts <- seq(stats::quantile(X, 0.05), stats::quantile(X, 0.95), length = k+2)[-c(1,k+2)]
  cuts0 <- c(min(X)-0.001, cuts, max(X)+0.001)
  data$x.cut <- cut(X, breaks = cuts0, include.lowest=TRUE, right=TRUE)
  formula.n <- stats::update(formula, stats::as.formula(". ~ . + x.cut"))
  fit <- mgcv::gam(formula.n, family = "binomial", data = data)
  auc0 <- compute.empirical.AUC(stats::predict(fit)[Y==1], stats::predict(fit)[Y==0])

  # Random nodes
  if(randnodes == TRUE){
  for(rep in 1:5) {
    auc1 = 0
    cuts1 = cuts0
    cuts1[2:(k+1)] <- sample(X,k)
    cuts1 <- sort(cuts1)
    if(length(unique(cuts1)) == length(cuts1)) {
      data$x.cut=cut(X, breaks = cuts1, include.lowest=TRUE, right=TRUE)
      if (min(table(data$x.cut)) >= min.p.cat) {
        fit <- mgcv::gam(formula.n, family="binomial", data = data)
        auc1 <- compute.empirical.AUC(stats::predict(fit)[Y==1], stats::predict(fit)[Y==0])}
      if (auc1 > auc0) {
        auc0=auc1; cuts0=cuts1; cuts0=sort(cuts0)}}}
  }

  # Sequential search setup
  caux <- seq(min(X), max(X), length.out = l.s.points + 2)[-c(1, l.s.points + 2)]
  nrep <- 0
  auc_old <- 0
  continue <- TRUE

  # Sequential search
  while (continue) {
    nrep = nrep+1
    for (j in 2:(k+1)) {
      for (ifino in 1:l.s.points) {
        auc1 = 0
        cuts1 = cuts0
        cuts1[j] = caux[ifino]
        cuts1 = sort(cuts1)
        if(length(cuts1) > length(unique(cuts1))) {auc1=0}
        else {
          data$x.cut <- cut(X, breaks = cuts1, include.lowest=TRUE, right=TRUE)
          fit <- mgcv::gam(formula.n, family = "binomial", data = data)
          if (min(table(data$x.cut)) >= min.p.cat) {
            auc1 <- compute.empirical.AUC(stats::predict(fit)[Y==1], stats::predict(fit)[Y==0])}
          }
        if (auc1 > auc0) {auc0=auc1; cuts0=cuts1; cuts0=sort(cuts0)}
        }}

      if (nrep > 1 & abs((auc0-auc_old)/auc_old) < eps) {continue=FALSE}
      if (nrep > repmax) {continue = FALSE}
      auc_old = auc0}

    data$x.cut <- cut(X, breaks = cuts0, include.lowest=TRUE, right=TRUE)
    fit <- mgcv::gam(formula.n, family = "binomial", data = data)
    list(cuts = cuts0[-c(1,k+2)], auc = auc0, pred = stats::predict(fit, type = "response"))
  }

k.points.max.cind <-
function(formula, cat.var, data, range, k, l.s.points = 100 , min.p.cat=1) {

	point=NULL
	mat=matrix(ncol=2,nrow=k)
	colnames(mat)<- c("point","c-index")

	for (i in 1:k){
		cinds <- select.cutpoint.cind(formula = formula, cat.var = cat.var, data = data, range = range, point = point, l.s.points = l.s.points, min.p.cat = min.p.cat)
		pos <- which(cinds[,2]==max(cinds[,2],na.rm=TRUE))
		if(length(pos)> 1 & i > 1) {
			if (cinds[pos[1],1] <= mat[i-1,1]) {
				pos <- max(pos)
			} else {
				pos = min(pos)
			}
		} else {
			pos = pos[1]
		}
		mat[i,1] <- cinds[pos,1]
		mat[i,2] <- cinds[pos,2]
		point <- c(point,mat[i,1])
	}
	mat
}

cindex.categorization <-
function (x, y) {
	if (!survival::is.Surv(y))
		y <- survival::Surv(y)

	i <- is.na(x) | is.na(y)
	if (any(i)) {
		x <- x[!i]
		y <- y[!i, ]
	}
	k <- survival::concordancefit(y, x)$count
	cindex <- (k[2] + (k[3]+k[4]+k[5])/2)/sum(k[1:5])
	cindex
}

#' @export
print.catpredi <-
function(x, digits = 4, ...) {
	cat("\nCall:\n")
	print(x$call)
	cat("\n\n*************************************************\n")
	cat("\n*************************************************\n")
	method <- switch(x$method,
	"addfor" = cat("Addfor Search Algorithm"),
	"genetic" = cat("Genetic Search Algorithm"),
	"backaddfor" = cat("Backaddfor Search Algorithm"))
	cat("\n*************************************************\n")
	cat("\n*************************************************\n\n")
  if(x$method == "addfor") {
	if(x$correct.AUC == TRUE){
	cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
		AUC  <- format(x$results$AUC, digits = digits, justify = "left")
		AUC.corrected <- format(x$results$AUC.cor, digits = digits, justify = "left")
		p.table <- cbind(cutpoints, AUC,c(rep("NA",length(x$results$cutpoints)-1),AUC.corrected))
		dimnames(p.table) <- list(rep("", l = length(x$results$AUC)), c("Optimal cutpoints", "Optimal AUC", "Corrected AUC"))
		print(p.table, quote = FALSE, justify = "right")
	} else {
		cutpoints <- format(x$results$cutpoints, digits = digits, justify = "right")
		AUC  <- format(x$results$AUC, digits = digits, justify = "left")
		p.table <- cbind(cutpoints, AUC)
		dimnames(p.table) <- list(rep("", l = length(x$results$AUC)), c("Optimal cutpoints", "Optimal AUC"))
		print(p.table, quote = FALSE, justify = "right")
	}
	} else { # Genetic/Backaddfor

	if(x$correct.AUC == TRUE){
	cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
		dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
		print(cutpoints, quote = FALSE, justify = "right")
		cat("\n")
		AUC <- cbind(format(x$results$AUC, digits = digits, justify = "right"))
		dimnames(AUC) <- list(rep("", l = length(x$results$AUC)), "Optimal AUC")
		print(AUC, quote = FALSE, justify = "right")
	cat("\n")
		AUC.corrected <- cbind(format(x$results$AUC.cor, digits = digits, justify = "right"))
		dimnames(AUC.corrected) <- list(rep("", l = length(x$results$AUC.cor)), "Corrected AUC")
		print(AUC.corrected, quote = FALSE, justify = "right")

	} else {
	cutpoints <- cbind(format(sort(x$results$cutpoints), digits = digits, justify = "right"))
		dimnames(cutpoints) <- list(rep("", l = length(x$results$cutpoints)), "Optimal cutpoints")
		print(cutpoints, quote = FALSE, justify = "right")
		cat("\n")
		AUC <- cbind(format(x$results$AUC, digits = digits, justify = "right"))
		dimnames(AUC) <- list(rep("", l = length(x$results$AUC)), "Optimal AUC")
		print(AUC, quote = FALSE, justify = "right")
		}
	}
	invisible(x)
}

backaddfor.cind <-
  function(formula, cat.var, data, range = NULL, k = 1, l.s.points = 100, eps = 0.001, repmax = 50, min.p.cat = 1, randnodes = FALSE){

    Y <- data[,all.vars(formula)[1]]
    if(is.null(range)) {
      X <- data[[cat.var]]
    } else{
      X <- data[[cat.var]][data[[cat.var]] >= range[1] & data[[cat.var]] <= range[2]]
    }
    var.names <- c(all.vars(formula), cat.var)

    # range is a vector such as: range=c(a,b)

    # Initial nodes
    cuts <- seq(stats::quantile(X, 0.05), stats::quantile(X, 0.95), length = k+2)[-c(1,k+2)]
    cuts0 <- c(min(X)-0.001, cuts, max(X)+0.001)
    cind.matrix <- matrix(ncol = 2, nrow = length(cuts0))
    colnames(cind.matrix)<- c("points","c-index")
    data$x.cut <- cut(X, breaks = cuts0, include.lowest=TRUE, right=TRUE)
    formula.n <- stats::update(formula, stats::as.formula(". ~ . + x.cut"))
    # fit <- gam(formula.n, family = "binomial", data = data)
    fit <- try(rms::cph(formula.n, data = data))
    # auc0 <- roc(Y, predict(fit), quiet=TRUE)$auc
    cind0 <- cindex.categorization(fit$linear.predictors, survival::Surv(data[,var.names[1]],data[,var.names[2]]))
    # cind0 <- cindex.categorization(fit$linear.predictors, Surv(data[,all.vars(formula.n)[1]], data[,all.vars(formula.n)[2]]))

    # Random nodes
    if(randnodes == TRUE){
      for(rep in 1:5) {
        cind1 = 0
        cuts1 = cuts0
        cuts1[2:(k+1)] <- sample(X,k)
        cuts1 <- sort(cuts1)
        if(length(unique(cuts1)) == length(cuts1)) {
          data$x.cut=cut(X, breaks = cuts1, include.lowest=TRUE, right=TRUE)
          if (min(table(data$x.cut)) >= min.p.cat) {
            fit <- try(rms::cph(formula.n, data = data))
            cind1 <- cindex.categorization(fit$linear.predictors, survival::Surv(data[,var.names[1]],data[,var.names[2]]))}
          if (cind1 > cind0) {
            cind0=cind1; cuts0=cuts1; cuts0=sort(cuts0)}}}
    }

    # Sequential search setup
    caux <- seq(min(X), max(X), length.out = l.s.points + 2)[-c(1, l.s.points + 2)]
    nrep <- 0
    cind_old <- 0
    continue <- TRUE

    # Sequential search
    while (continue) {
      nrep = nrep+1
      for (j in 2:(k+1)) {
        for (ifino in 1:l.s.points) {
          cind1 = 0
          cuts1 = cuts0
          cuts1[j] = caux[ifino]
          cuts1 = sort(cuts1)
          if(length(cuts1) > length(unique(cuts1))) {auc1=0}
          else {
            data$x.cut <- cut(X, breaks = cuts1, include.lowest=TRUE, right=TRUE)
            fit <- try(rms::cph(formula.n, data = data))
            # if (min(table(data$x.cut)) >= min.p.cat) {
            if(length(class(fit)) > 1){
              cind1 <- cindex.categorization(fit$linear.predictors, survival::Surv(data[,var.names[1]],data[,var.names[2]]))
            } else {cind1 <- NA}
          }
          if (cind1 > cind0) {cind0=cind1; cuts0=cuts1; cuts0=sort(cuts0)}
        }
      }

      if (nrep > 1 & abs((cind0-cind_old)/cind_old) < eps) {continue=FALSE}
      if (nrep > repmax) {continue = FALSE}
      cind_old = cind0}

    data$x.cut <- cut(X, breaks = cuts0, include.lowest=TRUE, right=TRUE)
    fit <- try(rms::cph(formula.n, data = data))
    list(cuts = cuts0[-c(1,k+2)], cind = cind0)
    # list(cuts = cuts0[-c(1,k+2)], cind = cind0, pred = predict(fit, type = "response"))
  }

#' A bootstrap-based hypothesis test to select the best number of categories for a
#'    continuous predictor variable in a logistic regression model
#'
#' @param obj1 An object inheriting from class \code{"catpredi"} for k number of cut points.
#' @param obj2 An object inheriting from class \code{"catpredi"} for k+1 number of cut points.
#' @param level The confidence level required for the hypothesis test. By default level = 0.95.
#' @param nb Number of bootstrap resamples. By default nb = 100
#' @param parallel A logical value. if TRUE the bootstrap is processed in parallel.
#' @param plot A logical value. if TRUE the density plot for the bootstrap statistic is provided.
#'
#' @returns
#' This function returns an object of class \code{"compare.AUC.ht"} with the following components:
#' \describe{
#'    \item{t.null}{test statistic, with the difference of the AUCs for the two objects.}
#'    \item{t.boot}{a vector with the \code{nb} bootstrap statistics.}
#'    \item{t.null}{empirical \code{level}-percentile of the bootstrap statistics vector.}
#' }
#'
#' @description Compares two objects of class "catpredi" to evaluate the significance
#'   of the improvement in model performance (in terms of the AUC) by adding k+1 cut-off
#'   points to the predictor variable.
#'
#' @references I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number
#'   of categories of the lymph node ratio in cancer research: A bootstrap-based
#'   hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
#'
#' @author Irantzu Barrio, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
#'
#' @seealso
#'  \code{\link{catpredi}},
#'  \code{\link{comp.cutpoints}}
#'
#' @importFrom foreach %dopar%
#'
#' @examples
#' library(CatPredi)
#' \dontrun{
#'   set.seed(127)
#'   #Simulate data
#'   n = 100
#'   #Predictor variable
#'   xh <- rnorm(n, mean = 0, sd = 1)
#'   xd <- rnorm(n, mean = 1.5, sd = 1)
#'   x <- c(xh, xd)
#'   #Response
#'   y <- c(rep(0,n), rep(1,n))
#'   # Data frame
#'   df <- data.frame(y = y, x = x)
#'   # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.addfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
#'                               data = df, method = "addfor", range=NULL, correct.AUC=TRUE,
#'                               control=controlcatpredi(grid=20))
#'   # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.addfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
#'                               data = df, method = "addfor", range=NULL, correct.AUC=TRUE,
#'                               control=controlcatpredi(grid=20))
#'     comp <-  comp.cutpoints(res.addfor.k2, res.addfor.k3, V = 10)
#'
#'   # Select 1 optimal cut points using the BackAddFor algorithm.
#'     res.backaddfor.k1 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 1,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
#'   # Select 2 optimal cut points using the BackAddFor algorithm.
#'     res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
#'   # Test if k=1 cut-off points is enough to categorise x
#'     comp.k1.k2 <-  compare.AUC.ht(res.backaddfor.k1, res.backaddfor.k2)
#' }
#'
#' @export
compare.AUC.ht <-
  function(obj1, obj2, level = 0.95, nb = 100, parallel = TRUE, plot = TRUE){
    if ((length(obj2$results$cutpoints) - length(obj1$results$cutpoints)) != 1) {
      stop(paste("Models (model 1 and model 2) do not have respectively", length(obj1$results$cutpoints), "and", length(obj1$results$cutpoints) + 1 ,"cut-off points."))
    }
    if (identical(obj1$formula, obj2$formula)){
      formula.ht <- obj1$formula
    } else {
      stop("The formulas of the models are not identical. Please verify that both formulas are the same.")
    }
    if (identical(obj1$cat.var, obj2$cat.var)) {
      cat.var.ht <- obj1$cat.var
    } else {
      stop("The variable to categorise is not the same in both models. Please ensure you are using the same variables.")
    }
    if (identical(obj1$data[, -ncol(obj1$data)],obj2$data[, -ncol(obj2$data)])) {
      data.ht <- obj1$data
    } else {
      stop("The datasets are not the same in both models. Please make sure you are using the same data.")
    }
    if (identical(obj1$method, obj2$method)) {
      method.ht <- obj1$method
    } else {
      stop("The methods are not the same in both models. Please ensure you are using the same method.")
    }
    if (identical(obj1$range, obj2$range)) {
      range.ht <- obj1$range
    } else {
      stop("Range argument is not the same in both models. Please verify they are specified in the same way.")
    }
    if (identical(obj1$correct.AUC, obj2$correct.AUC)) {
      correct.AUC.ht <- obj1$correct.AUC
    } else {
      stop("The correct.AUC argument is not the same in both models. Please verify they are defined in the same way.")
    }

    Tstatistic <- obj2$results$AUC - obj1$results$AUC

    nc <- length(obj1$results$cutpoints)
    variables <- all.vars(formula.ht)
    # res <- catpredi(formula = formula, cat.var = cat.var, cat.points = nc, data, method = method, range = range, control = controlcatpredi())
    formula.ht.n <- stats::update(formula.ht, stats::as.formula(paste0("~ . +", cat.var.ht, "_CatPredi")))
    model <- mgcv::gam(formula.ht.n, data = data.ht, family = "binomial")
    p.boot <- stats::predict(model, data.ht, type = "response")

    if (parallel) {
      n.cores <- max(1, parallel::detectCores() - 1)

      tryCatch({
        doParallel::registerDoParallel(n.cores)
      }, error = function(e) {
        message("Parallel setup failed: ", conditionMessage(e))
        message("Setting number of cores to 2.")
        n.cores <- 2
        doParallel::registerDoParallel(n.cores)
      })

    } else {
      n.cores <- 1
      doParallel::registerDoParallel(n.cores)
    }

    # Bootstrap
    df1.all <- foreach::foreach(i = 1:nb, .verbose = FALSE, .errorhandling = "stop", .inorder=FALSE, .combine = c) %dopar% {
      y <- stats::rbinom(n = nrow(data.ht), size = 1, prob = p.boot)
      data.ht$y <- y
      formula.n <- stats::update(formula.ht, stats::as.formula("y ~ ."))
      df1 <- catpredi(formula = formula.n, cat.var=cat.var.ht, cat.points = nc+1, data=data.ht, method = method.ht, range = range.ht, control = controlcatpredi())$results$AUC[length(catpredi(formula=formula.n, cat.var=cat.var.ht, cat.points = nc+1, data=data.ht, method = method.ht, range = range.ht, control = controlcatpredi())$results$AUC)]-
        catpredi(formula = formula.n, cat.var=cat.var.ht, cat.points = nc, data=data.ht, method = method.ht, range = range.ht, control = controlcatpredi())$results$AUC[length(catpredi(formula=formula.n, cat.var=cat.var.ht, cat.points = nc, data=data.ht, method = method.ht, range = range.ht, control = controlcatpredi())$results$AUC)]
      return(df1)
      gc()
    }
    # Stop the cluster
    doParallel::stopImplicitCluster()

    if(plot){
      plot(stats::density(df1.all), xlab = "t.boot", ylab = "", xlim = range(Tstatistic, stats::density(df1.all)$x),
           main = paste0("Formula = ", as.expression(formula.ht.n), "\n", nc, " vs ", nc+1, " cut-off points"))
      t <- stats::quantile(df1.all, level)
      graphics::abline(v = t, lty = 1)
      graphics::abline(v = Tstatistic, lty = 2)
      graphics::legend("topleft", legend = c(expression(t[level]),expression(t[null])), lty = 1:2, bty = "n", cex=1.5)
    }

    cat("\n\n*************************************************\n")
    cat(paste0("Bootstrap-based hypothesis test for ", nc, " vs ", nc+1, " cut points"))
    cat("\n*************************************************\n\n")

    if (Tstatistic <= t) {
      cat(paste("Don't reject the null hypothesis (H0). Thus, it is enough to take k =", nc, sep = " "))
    } else {
      cat(paste("Reject the null hypothesis (H0). Thus, it is advisable to take k =", nc + 1, sep = " "))
    }

    res<- list(t.null = Tstatistic, t.boost = df1.all, t.null = Tstatistic, t.alpha = t)
    class(res) <- "compare.AUC.ht"
    res
  }

cpe.opt.corrected <-
function(formula, cat.var, data, c.points, cpe , B , b.method = c("ncoutcome","coutcome")) {
	b.method <- match.arg(b.method)
  cpe.boot <- cpe.original  <- vector(length=B)
	var.names <- c(all.vars(formula), cat.var)
	X <- data[,cat.var]
	data.o <- data

	for (i in 1:B) {
		data.b <- bootstrap.sample(data, group = var.names[2],  method = b.method)
		sel.point = sort(unique(c(min(X, data.b[,cat.var]), max(X, data.b[,cat.var]), c.points)))
		x.cut.boot <- cut(data.b[,cat.var], sel.point, include.lowest = TRUE, right = TRUE)
		x.cut <-  cut(X, sel.point, include.lowest = TRUE, right = TRUE)
		if(length(levels(x.cut.boot)) == length(levels(x.cut)) & all(table(x.cut.boot)>1)){
			data.b[,"x.cut_"] <- x.cut.boot
			data.o[,"x.cut_"] <- x.cut
			# Bootstrap
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
			f.boot <- try(rms::cph(formula.n, data=data.b))
			#if(class(f.boot) == "try-error") {
			if("try-error" %in% class(f.boot)){
				cpe.boot[i] <- NA
				cpe.original[i] <- NA
			} else {
				# cpe.boot[i] <- cpe(f.boot, data.b)
			  cpe.boot[i] <- CPE::phcpe2(coef = f.boot$coefficients, coef.var = f.boot$var, design = stats::model.matrix(f.boot, data = data.b))$CPE  #phcpe(f.boot,CPE.SE=FALSE, out.ties=FALSE)$CPE

				# Original Sample
				# cpe.original[i] <- cpe(f.boot, data.o)
			  cpe.original[i] <- CPE::phcpe2(coef = f.boot$coefficients, coef.var = f.boot$var, design = stats::model.matrix(f.boot, data = data.o))$CPE
			}
		} else {
			cpe.boot[i] <- NA
			cpe.original[i] <- NA
		}
	}
	cpe.corrected <- cpe - mean(abs(cpe.original - cpe.boot), na.rm=TRUE)
	cpe.corrected
}

#' Selection of optimal number of cut points
#'
#' @param obj1 An object inheriting from class \code{"catpredi"} for k number of cut points
#' @param obj2 An object inheriting from class \code{"catpredi"} for k+1 number of cut points
#' @param V Number of bootstrap resamples. By default V=100
#'
#' @returns
#' This function returns an object of class \code{"comp.cutpoints"} with the following components:
#' \describe{
#'   \item{AUC.cor.diff}{the difference of the bias corrected AUCs for the two categorical variables.}
#'   \item{icb.auc.diff}{bootstrap based confidence interval for the bias corrected AUC difference.}
#' }
#'
#' @description Compares two objects of class \code{"catpredi"}.
#'
#' @references I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
#'    A new approach to categorising continuous variables in prediction models:
#'    proposal and validation. \emph{Statistical Methods in Medical Research},
#'    26(6), 2586-2602.
#'
#' @author Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
#'
#' @seealso
#' \code{\link{catpredi}}
#'
#' @examples
#' library(CatPredi)
#' set.seed(127)
#' #Simulate data
#' n = 100
#' #Predictor variable
#' xh <- rnorm(n, mean = 0, sd = 1)
#' xd <- rnorm(n, mean = 1.5, sd = 1)
#' x <- c(xh, xd)
#' #Response
#' y <- c(rep(0,n), rep(1,n))
#' # Data frame
#' df <- data.frame(y = y, x = x)
#' \dontshow{
#'   # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
#'                                   control=controlcatpredi(grid=20))
#'   # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.backaddfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
#'                                   control=controlcatpredi(grid=20))
#'     comp <-  comp.cutpoints(res.backaddfor.k2, res.backaddfor.k3, V = 10)
#'  }
#' \donttest{
#'   # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
#'                                   control=controlcatpredi(grid=100))
#'   # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
#'     res.backaddfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
#'                                   data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
#'                                   control=controlcatpredi(grid=100))
#'
#'   # Select optimal number of cut points
#'     comp <-  comp.cutpoints(res.backaddfor.k2, res.backaddfor.k3, V = 100)
#'  }
#'
#' @export
comp.cutpoints <-
function(obj1, obj2, V = 100) {
	if(is.null(obj2$results$AUC.cor)==TRUE || is.null(obj1$results$AUC.cor)==TRUE) {
		stop("argument correct.AUC=TRUE is needed in catpredi")
	}
	if(obj1$formula != obj2$formula) {
		stop("The categorized variables are not comparable")
	}
	if(obj1$control$B != obj2$control$B) {
		warning("The bootstrap resamples used for the optimism correction of the AUC is different in both objects")
	}
	AUC.cor.diff <- obj2$results$AUC.cor - obj1$results$AUC.cor
	formula <- obj1$formula

	point1 <- obj1$results$cutpoints
	point2 <- obj2$results$cutpoints
	B1 <-   obj1$control$B
  B2 <-   obj2$control$B
  b.method <- obj1$control$b.method
	data <-   obj1$data
	cat.var <- obj1$cat.var
	var.names <- all.vars(formula)
	X <- data[,cat.var]
	Y <- data[,var.names[1]]

	auc.b.1 <- auc.b.2 <- auc.b.diff <- vector(length = V)

	for (i in 1:V) {
		data.b <- bootstrap.sample(data, group = var.names[1], method = b.method)
		Y.b <- data.b[,var.names[1]]
		X.b <- data.b[,cat.var]
		# k
		sel.point = sort(unique(c(min(X,X.b , na.rm=TRUE), max(X,X.b , na.rm=TRUE), point1)))
		x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
		data.b[,"x.cut1"] <- x.cut
		formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut1"))
			fit.1 <- mgcv::gam(formula.n, family = stats::binomial, data = data.b)
		auc.fit1 <- compute.empirical.AUC(fit.1$fitted[Y.b==1], fit.1$fitted[Y.b==0])
		auc.b.1[i] <- auc.opt.corrected(formula = formula, cat.var = cat.var, data = data.b, c.points = point1, AUC =auc.fit1 , B=B1, b.method = b.method)

		# k = k+1
		sel.point = sort(unique(c(min(X,X.b , na.rm=TRUE), max(X,X.b , na.rm=TRUE), point2)))
		x.cut = cut(X.b, sel.point, include.lowest = TRUE, right = TRUE)
		data.b[,"x.cut2"] <- x.cut
		formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut2"))
		fit.2 <- mgcv::gam(formula.n, family = stats::binomial, data = data.b)
		auc.fit2 <- compute.empirical.AUC(fit.2$fitted[Y.b==1], fit.2$fitted[Y.b==0])
		auc.b.2[i] <- auc.opt.corrected(formula = formula, cat.var = cat.var, data = data.b, c.points = point2, AUC =auc.fit2 , B=B2, b.method = b.method)
		auc.b.diff[i] <- auc.b.2[i] - auc.b.1[i]
	}
	auc.diff <- stats::quantile(auc.b.diff, p = c(0.025, 0.975), na.rm=TRUE)
	res <- list( AUC.cor.diff = AUC.cor.diff , icb.auc.diff = auc.diff)
	class(res) <- "comp.cutpoints"
  res
}

select.cutpoint.cind <-
function(formula, cat.var, data , range, point, l.s.points = 100, min.p.cat) {
	search.points=seq(min(range),max(range),l=l.s.points)
	cind.matrix=matrix(ncol=2,nrow=length(search.points))
	colnames(cind.matrix)<- c("points","c-index")
	var.names <- c(all.vars(formula), cat.var)
	X <- data[,cat.var]

	for (i in 1:length(search.points)){
		cutoffs=sort(unique(c(min(X), max(X),point, search.points[i])))
		x.cut=cut(X,cutoffs, include.lowest=TRUE,right=TRUE)
		if(length(levels(x.cut)) > 1 & all(table(x.cut)>min.p.cat)) {
			data[,"x.cut_"] <- x.cut
			formula.n <- stats::update(formula, stats::as.formula("~ . + x.cut_"))
			fit <- try(rms::cph(formula.n, data = data))
			#if(class(fit) == "try-error"){
			if("try-error" %in% class(fit)) {
				cind.matrix[i,1] <- search.points[i]
				cind.matrix[i,2] <- NA
			} else {
				cind.matrix[i,1] <- search.points[i]
				cind.matrix[i,2] <- cindex.categorization(fit$linear.predictors, survival::Surv(data[,var.names[1]],data[,var.names[2]]))
			}
		} else {
			cind.matrix[i,1] <- search.points[i]
			cind.matrix[i,2] <- NA
	 	}
	}
	cind.matrix
}

#' @export
print.comp.cutpoints.survival <-
function(x, digits = 4, ...) {
  cat("\nCall:\n")
	print(x$call)
	cat("\n\n*************************************************\n")
  cat("Compare optimal number of cut points")
	cat("\n*************************************************\n\n")
	cat(paste("Bias corrected concordance difference:", round(x$CI.cor.diff, 4), sep="  "), fill=TRUE)
	cat(paste("95% Bootstrap Confidence Interval:","(",round(x$icb.CI.diff[1], 4),",", round(x$icb.CI.diff[2], 4), ")" ,sep=" "), fill=TRUE)
  invisible(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp.cutpoints.R
\name{comp.cutpoints}
\alias{comp.cutpoints}
\title{Selection of optimal number of cut points}
\usage{
comp.cutpoints(obj1, obj2, V = 100)
}
\arguments{
\item{obj1}{An object inheriting from class \code{"catpredi"} for k number of cut points}

\item{obj2}{An object inheriting from class \code{"catpredi"} for k+1 number of cut points}

\item{V}{Number of bootstrap resamples. By default V=100}
}
\value{
This function returns an object of class \code{"comp.cutpoints"} with the following components:
\describe{
\item{AUC.cor.diff}{the difference of the bias corrected AUCs for the two categorical variables.}
\item{icb.auc.diff}{bootstrap based confidence interval for the bias corrected AUC difference.}
}
}
\description{
Compares two objects of class \code{"catpredi"}.
}
\examples{
library(CatPredi)
set.seed(127)
#Simulate data
n = 100
#Predictor variable
xh <- rnorm(n, mean = 0, sd = 1)
xd <- rnorm(n, mean = 1.5, sd = 1)
x <- c(xh, xd)
#Response
y <- c(rep(0,n), rep(1,n))
# Data frame
df <- data.frame(y = y, x = x)
\dontshow{
  # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
    res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
                                  control=controlcatpredi(grid=20))
  # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
    res.backaddfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
                                  control=controlcatpredi(grid=20))
    comp <-  comp.cutpoints(res.backaddfor.k2, res.backaddfor.k3, V = 10)
 }
\donttest{
  # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
    res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
                                  control=controlcatpredi(grid=100))
  # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
    res.backaddfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=TRUE,
                                  control=controlcatpredi(grid=100))

  # Select optimal number of cut points
    comp <-  comp.cutpoints(res.backaddfor.k2, res.backaddfor.k3, V = 100)
 }

}
\references{
I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
A new approach to categorising continuous variables in prediction models:
proposal and validation. \emph{Statistical Methods in Medical Research},
26(6), 2586-2602.
}
\seealso{
\code{\link{catpredi}}
}
\author{
Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp.cutpoints.survival.R
\name{comp.cutpoints.survival}
\alias{comp.cutpoints.survival}
\title{Selection of optimal number of cut points}
\usage{
comp.cutpoints.survival(obj1, obj2, V = 100)
}
\arguments{
\item{obj1}{An object inheriting from class \code{"catpredi.survival"} for k number of cut points}

\item{obj2}{An object inheriting from class \code{"catpredi.survival"} for k+1 number of cut points}

\item{V}{Number of bootstrap resamples. By default V=100}
}
\value{
This function returns an object of class "comp.cutpoints.survival" with the following components:
\describe{
\item{CI.cor.diff}{the difference of the bias corrected concordance probability for the two categorical variables.}
\item{icb.CI.diff}{bootstrap based confidence interval for the bias corrected concordance probability difference.}
}
}
\description{
Compares two objects of class \code{"catpredi.survival"}
}
\examples{
library(CatPredi)
library(survival)
set.seed(123)

#Simulate data
n = 300
tauc = 1
X <- rnorm(n=n, mean=0, sd=2)
SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
# Censoring time
CensTime <- runif(n=n, min=0, max=tauc)
# Status
SurvS <- as.numeric(SurvT <= CensTime)
# Data frame
dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)
\donttest{
  # Select 2 optimal cut points using the AddFor algorithm. Correct the c-index
    res.k2 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
                                 data = dat, method = "addfor", conc.index = "cindex",
                                 range = NULL, correct.index = TRUE)
  # Select 3 optimal cut points using the AddFor algorithm. Correct the c-index
    res.k3 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 3,
                                 data = dat, method = "addfor", conc.index = "cindex",
                                 range = NULL, correct.index = TRUE)
  # Select optimal number of cut points
    comp <-  comp.cutpoints.survival(res.k2, res.k3, V = 100)
}
\dontshow{
  # Select 2 optimal cut points using the AddFor algorithm. Correct the c-index
    res.k2 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 1,
                                 data = dat, method = "addfor", conc.index = "cindex",
                                 range = NULL, correct.index = TRUE,
                                 control=controlcatpredi.survival(grid=20))
  # Select 3 optimal cut points using the AddFor algorithm. Correct the c-index
    res.k3 <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
                                 data = dat, method = "addfor", conc.index = "cindex",
                                 range = NULL, correct.index = TRUE,
                                 control=controlcatpredi.survival(grid=20))
  # Select optimal number of cut points
    comp <-  comp.cutpoints.survival(res.k2, res.k3, V = 2)
}

}
\references{
I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and  I Arostegui (2017).
Comparison of two discrimination indexes in the categorisation of continuous predictors in time-to-event studies.
\emph{SORT}, 41:73-92
}
\seealso{
\code{\link{catpredi.survival}}
}
\author{
Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlcatpredi.R
\name{controlcatpredi}
\alias{controlcatpredi}
\title{Control function}
\usage{
controlcatpredi(
  min.p.cat = 1,
  grid = 100,
  B = 50,
  eps = 0.001,
  b.method = c("ncoutcome", "coutcome"),
  print.gen = 0
)
}
\arguments{
\item{min.p.cat}{Set the minimun number of individuals in each category}

\item{grid}{Grid size for the AddFor and BackAddFor algorithms}

\item{B}{Number of bootstrap replicates for the AUC bias correction procedure}

\item{eps}{An argument for the BackAddFor algorithm, indicating whether the
improvement between iterations is considered significant}

\item{b.method}{Allows to specify whether the bootstrap resampling should be
done considering or not the outcome variable.
The option "ncoutcome" indicates that the data is resampled without taking
into account the response variable, while "coutcome" indicates that the data
is resampled in regard to the response variable}

\item{print.gen}{Corresponds to the argument print.level of the \code{\link[rgenoud]{genoud}}
function of the package \pkg{rgenoud}.}
}
\value{
A list with components for each of the possible arguments.
}
\description{
Function used to set several parameters to control the selection of
the optimal cut points in a logistic regression model
}
\references{
Mebane Jr, W. R., & Sekhon, J. S. (2011). Genetic optimization using derivatives:
the rgenoud package for R. \emph{Journal of Statistical Software} 42\bold{11}, 1-26.
}
\seealso{
\code{\link{catpredi}}
}
\author{
Irantzu Barrio, Maria Xose Rodriguez-Alvarez, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.catpredi.survival.R
\name{summary.catpredi.survival}
\alias{summary.catpredi.survival}
\title{Summary method for objects of class "catpredi.survival"}
\usage{
\method{summary}{catpredi.survival}(object, digits = 4, ...)
}
\arguments{
\item{object}{An object of class "catpredi.survival" as produced by catpredi.survival()}

\item{digits}{.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Returns an object of class "summary.catpredi.survival" with the same components
as the catpredi.survival function (see \code{\link{catpredi.survival}}).
}
\description{
Produces a summary of a "catpredi.survival" object. The following
are printed: the call to the catpredi.survival() function; the estimated
optimal cut points obtained with the method and concordance probability
estimator selected and the estimated and bias corrected concordance probability
for the categorised variable (whenever the argument correct.index is set to TRUE).
}
\examples{
library(CatPredi)
library(survival)
set.seed(123)
#Simulate data
n = 500
tauc = 1
X <- rnorm(n=n, mean=0, sd=2)
SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
# Censoring time
CensTime <- runif(n=n, min=0, max=tauc)
# Status
SurvS <- as.numeric(SurvT <= CensTime)
# Data frame
dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)

# Select optimal cut points using the AddFor algorithm
res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
                          data = dat, method = "addfor", conc.index = "cindex", range = NULL,
                          correct.index = FALSE)
# Summary
summary(res)

}
\references{
I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92
}
\seealso{
\code{\link{catpredi.survival}}
}
\author{
Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.catpredi.survival.R
\name{plot.catpredi.survival}
\alias{plot.catpredi.survival}
\title{Plot the optimal cut points.}
\usage{
\method{plot}{catpredi.survival}(x, ...)
}
\arguments{
\item{x}{An object of type catpredi.survival.}

\item{...}{Additional arguments to be passed on to other functions. Not yet implemented.}
}
\value{
This function returns the plot of the relationship between the predictor variable and the outcome.
}
\description{
Plots the functional form of the predictor variable we want to
categorise. Additionally, the optimal cut points obtained with the
catpredi.survival() function are drawn on the graph.
}
\examples{
library(CatPredi)
library(survival)
set.seed(123)
#Simulate data
n = 500
tauc = 1
X <- rnorm(n=n, mean=0, sd=2)
SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
# Censoring time
CensTime <- runif(n=n, min=0, max=tauc)
# Status
SurvS <- as.numeric(SurvT <= CensTime)
# Data frame
dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)

# Select optimal cut points using the AddFor algorithm
res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
                          data = dat, method = "addfor", conc.index = "cindex", range = NULL,
                          correct.index = FALSE)
# Plot
plot(res)

}
\references{
I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92
}
\seealso{
\code{\link{catpredi.survival}}
}
\author{
Irantzu Barrio and Maria Xose Rodriguez-Alvarez.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catpredi.survival.R
\name{catpredi.survival}
\alias{catpredi.survival}
\title{Function to obtain optimal cut points to categorise a continuous predictor variable
in a Cox proportional hazards regression model}
\usage{
catpredi.survival(
  formula,
  cat.var,
  cat.points = 1,
  data,
  method = c("addfor", "genetic", "backaddfor"),
  conc.index = c("cindex", "cpe"),
  range = NULL,
  correct.index = FALSE,
  control = controlcatpredi.survival(),
  ...
)
}
\arguments{
\item{formula}{An object of class \code{\link{formula}} giving the model to be
fitted in addition to the continuous covariate is aimed to categorise.
The response must be a survival object as returned by the \code{\link[survival]{Surv}}
function. This argument allows the user to specify whether the continuous
predictor should be categorised in a univariable context, or in presence of
other covariates or cofounders, i.e in a multiple Cox proportional hazards
regression model. For instance, Surv(SurvT,SurvS)~1 indicates that the
categorisation should be done in a univariable setting.}

\item{cat.var}{Name of the continuous variable to categorise.}

\item{cat.points}{Number of cut points to look for.}

\item{data}{Data frame containing all needed variables.}

\item{method}{The algorithm selected to search for the optimal cut points.
\code{"addfor"} if the AddFor algorithm is choosen, \code{"backaddfor"} if
the BackAddFor algorithm is selected and \code{"genetic"} otherwise.}

\item{conc.index}{The concordance probability estimator selected for maximisation
purposes. "cindex" if the c-index concordance probability is choosen and
"cpe" otherwise. The c-index and CPE are estimated using the \pkg{rms} and
\pkg{CPE} packages, respectively.}

\item{range}{The range of the continuous variable in which to look for the cut
points. By default \code{NULL}, i.e, all the range.}

\item{correct.index}{A logical value. If TRUE the bias corrected concordance
probability is estimated.}

\item{control}{Output of the \code{\link{controlcatpredi.survival}} function.}

\item{...}{Further arguments for passing on to the function \code{\link[rgenoud]{genoud}}
of the package \pkg{rgenoud}.}
}
\value{
Returns an object of class \code{"catpredi.survival"} with the following components:
\describe{
\item{call}{The matched call.}
\item{method}{The algorithm selected in the call.}
\item{formula}{an object of class \code{\link{formula}} giving the model to
be fitted in addition to the continuous covariate is aimed to categorise.}
\item{cat.var}{name of the continuous variable to categorise.}
\item{data}{the data frame with the variables used in the call.}
\item{correct.index}{The logical value used in the call.}
\item{results}{a list with the estimated cut points, concordance probability
and bias corrected concordance probability.}
\item{control}{the control parameters used in the call.}
When the c-index concordance probability is choosen, a list with the following
components is obtained for each of the methods used in the call:
\item{"cutpoints"}{Estimated optimal cut points.}
\item{"Cindex"}{Estimated c-index.}
\item{"Cindex.cor"}{Estimated bias corrected c-index.}
When the CPE concordance probability is choosen, a list with the following
components is obtained for each of the methods used in the call:
\item{"cutpoints"}{Estimated optimal cut points.}
\item{"CPE"}{Estimated CPE.}
\item{"CPE.cor"}{Estimated bias corrected CPE.}
}
}
\description{
Returns an object with the optimal cut points to categorise a continuous
predictor variable in a Cox proportional hazards regression model
}
\examples{
library(CatPredi)
library(survival)
set.seed(123)
#Simulate data
n = 500
tauc = 1
X <- rnorm(n=n, mean=0, sd=2)
SurvT <- exp(2*X + rweibull(n = n, shape=1, scale = 1))   + rnorm(n, mean=0, sd=0.25)
# Censoring time
CensTime <- runif(n=n, min=0, max=tauc)
# Status
SurvS <- as.numeric(SurvT <= CensTime)
# Data frame
dat <- data.frame(X = X, SurvT = pmin(SurvT, CensTime), SurvS = SurvS)

# Select optimal cut points using the AddFor algorithm
res <- catpredi.survival (formula= Surv(SurvT,SurvS)~1, cat.var="X", cat.points = 2,
                          data = dat, method = "addfor", conc.index = "cindex", range = NULL,
                          correct.index = FALSE)

}
\references{
I Barrio,  M.X Rodriguez-Alvarez, L Meira-Machado, C Esteban  and
I Arostegui (2017). Comparison of two discrimination indexes in the categorisation
of continuous predictors in time-to-event studies. \emph{SORT}, 41:73-92

M Gonen and  G Heller (2005). Concordance probability and discriminatory power
in  proportional hazards regression. \emph{Biometrika}, 92:965-970.

F Harrell (2001). Regression modeling strategies: with applications to linear models,
logistic and ordinal regression, and survival analysis. Springer.
}
\seealso{
\code{\link{controlcatpredi.survival}},
\code{\link{comp.cutpoints.survival}},
\code{\link{plot.catpredi.survival}},
\code{\link{catpredi}}
}
\author{
Irantzu Barrio and Maria Xose Rodriguez-Alvarez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.AUC.ht.R
\name{compare.AUC.ht}
\alias{compare.AUC.ht}
\title{A bootstrap-based hypothesis test to select the best number of categories for a
continuous predictor variable in a logistic regression model}
\usage{
compare.AUC.ht(
  obj1,
  obj2,
  level = 0.95,
  nb = 100,
  parallel = TRUE,
  plot = TRUE
)
}
\arguments{
\item{obj1}{An object inheriting from class \code{"catpredi"} for k number of cut points.}

\item{obj2}{An object inheriting from class \code{"catpredi"} for k+1 number of cut points.}

\item{level}{The confidence level required for the hypothesis test. By default level = 0.95.}

\item{nb}{Number of bootstrap resamples. By default nb = 100}

\item{parallel}{A logical value. if TRUE the bootstrap is processed in parallel.}

\item{plot}{A logical value. if TRUE the density plot for the bootstrap statistic is provided.}
}
\value{
This function returns an object of class \code{"compare.AUC.ht"} with the following components:
\describe{
\item{t.null}{test statistic, with the difference of the AUCs for the two objects.}
\item{t.boot}{a vector with the \code{nb} bootstrap statistics.}
\item{t.null}{empirical \code{level}-percentile of the bootstrap statistics vector.}
}
}
\description{
Compares two objects of class "catpredi" to evaluate the significance
of the improvement in model performance (in terms of the AUC) by adding k+1 cut-off
points to the predictor variable.
}
\examples{
library(CatPredi)
\dontrun{
  set.seed(127)
  #Simulate data
  n = 100
  #Predictor variable
  xh <- rnorm(n, mean = 0, sd = 1)
  xd <- rnorm(n, mean = 1.5, sd = 1)
  x <- c(xh, xd)
  #Response
  y <- c(rep(0,n), rep(1,n))
  # Data frame
  df <- data.frame(y = y, x = x)
  # Select 2 optimal cut points using the AddFor algorithm. Correct the AUC
    res.addfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
                              data = df, method = "addfor", range=NULL, correct.AUC=TRUE,
                              control=controlcatpredi(grid=20))
  # Select 3 optimal cut points using the AddFor algorithm. Correct the AUC
    res.addfor.k3 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
                              data = df, method = "addfor", range=NULL, correct.AUC=TRUE,
                              control=controlcatpredi(grid=20))
    comp <-  comp.cutpoints(res.addfor.k2, res.addfor.k3, V = 10)

  # Select 1 optimal cut points using the BackAddFor algorithm.
    res.backaddfor.k1 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 1,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
  # Select 2 optimal cut points using the BackAddFor algorithm.
    res.backaddfor.k2 <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 2,
                                  data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
  # Test if k=1 cut-off points is enough to categorise x
    comp.k1.k2 <-  compare.AUC.ht(res.backaddfor.k1, res.backaddfor.k2)
}

}
\references{
I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number
of categories of the lymph node ratio in cancer research: A bootstrap-based
hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
}
\seealso{
\code{\link{catpredi}},
\code{\link{comp.cutpoints}}
}
\author{
Irantzu Barrio, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.catpredi.R
\name{summary.catpredi}
\alias{summary.catpredi}
\title{Summary method for catpredi objects}
\usage{
\method{summary}{catpredi}(object, digits = 4, ...)
}
\arguments{
\item{object}{An object of class catpredi as produced by catpredi()}

\item{digits}{.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Returns an object of class "summary.catpredi" with the same components as the
catpredi function (see \code{\link{catpredi}}). plus:
\describe{
\item{fit.gam}{fitted model according to the model specified in the call,
based on the function \code{\link[mgcv]{gam}} of the package \pkg{mgcv}.}
}
}
\description{
Produces a summary of a catpredi object. The following are printed:
the call to the catpredi() function; the estimated optimal cut points obtained
with the method selected and the estimated AUC and bias corrected AUC (if the
argument correct.AUC is TRUE) for the categorised variable.
}
\examples{
library(CatPredi)
set.seed(127)
#Simulate data
n = 200
#Predictor variable
xh <- rnorm(n, mean = 0, sd = 1)
xd <- rnorm(n, mean = 1.5, sd = 1)
x <- c(xh, xd)
#Response
y <- c(rep(0,n), rep(1,n))
#Covariate
zh <- rnorm(n, mean=1.5, sd=1)
zd <- rnorm(n, mean=1, sd=1)
z <- c(zh, zd)
# Data frame
df <- data.frame(y = y, x = x, z = z)

# Select optimal cut points using the AddFor algorithm
res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 2,
                           data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
# Summary
summary(res.backaddfor)

}
\references{
I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
A new approach to categorising continuous variables in prediction models:
proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.

I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number of
categories of the lymph node ratio in cancer research: A bootstrap-based
hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
}
\seealso{
\code{\link{catpredi}}
}
\author{
Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.catpredi.R
\name{plot.catpredi}
\alias{plot.catpredi}
\title{Plot the optimal cut points.}
\usage{
\method{plot}{catpredi}(x, ...)
}
\arguments{
\item{x}{An object of type catpredi.}

\item{...}{Additional arguments to be passed on to other functions. Not yet implemented.}
}
\value{
This function returns the plot of the relationship between the predictor variable and the outcome.
}
\description{
Plots the relationship between the predictor variable is aimed to
categorise and the response variable based on a GAM model.
Additionally, the optimal cut points obtained with the catpredi() function
are drawn on the graph.
}
\examples{
 library(CatPredi)
 \dontrun{
 set.seed(127)
 #Simulate data
 n = 100
 #Predictor variable
 xh <- rnorm(n, mean = 0, sd = 1)
 xd <- rnorm(n, mean = 1.5, sd = 1)
 x <- c(xh, xd)
 #Response
 y <- c(rep(0,n), rep(1,n))
 # Data frame
 df <- data.frame(y = y, x = x)

 # Select optimal cut points using the AddFor algorithm
 res.backaddfor <- catpredi(formula = y ~ 1, cat.var = "x", cat.points = 3,
                            data = df, method = "backaddfor", range = NULL, correct.AUC = FALSE)
 # Plot
 plot(res.backaddfor)
 }
}
\references{
I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
A new approach to categorising continuous variables in prediction models:
proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.

I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number of
categories of the lymph node ratio in cancer research: A bootstrap-based
hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.
}
\seealso{
\code{\link{catpredi}}
}
\author{
Irantzu Barrio, Maria Xose Rodriguez-Alvarez and Inmaculada Arostegui.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catpredi.R
\name{catpredi}
\alias{catpredi}
\title{Function to obtain optimal cut points to categorise a continuous predictor
variable in a logistic regression model}
\usage{
catpredi(
  formula,
  cat.var,
  cat.points = 1,
  data,
  method = c("addfor", "genetic", "backaddfor"),
  range = NULL,
  correct.AUC = FALSE,
  control = controlcatpredi(),
  ...
)
}
\arguments{
\item{formula}{An object of class \code{\link{formula}} giving the model to be
fitted in addition to the continuous covariate is aimed to categorise.
This argument allows the user to specify whether the continuous predictor
should be categorised in a univariable context, or in presence of other
covariates or cofounders, i.e in a multiple logistic regression model.
For instance, \code{Y ~ 1} indicates that the categorisation should be done in a
univariable setting, with Y being the response variable. If the predictor
variable is aimed to be categorised in a multivariable setting, this argument
allows to specify whether the covariates should be modelled using linear
or non linear effects. In the latest, the effects are estimated using the
\pkg{mgcv} package.}

\item{cat.var}{Name of the continuous variable to categorise.}

\item{cat.points}{Number of cut points to look for.}

\item{data}{Data frame containing all needed variables.}

\item{method}{The algorithm selected to search for the optimal cut points.
\code{"addfor"} if the AddFor algorithm is choosen, \code{"backaddfor"} if
the BackAddFor algorithm is selected and \code{"genetic"} otherwise.}

\item{range}{The range of the continuous variable in which to look for the cut
points. By default \code{NULL}, i.e, all the range.}

\item{correct.AUC}{A logical value. If \code{TRUE} the bias corrected AUC is estimated.}

\item{control}{Output of the \code{\link{controlcatpredi}} function.}

\item{...}{Further arguments for passing on to the function \code{\link[rgenoud]{genoud}}
of the package \pkg{rgenoud}.}
}
\value{
Returns an object of class \code{"catpredi"} with the following components:
\describe{
\item{call}{The matched call.}
\item{method}{The algorithm selected in the call.}
\item{formula}{The model formula used in the call.}
\item{cat.var}{Name of the continuous variable to categorise.}
\item{data}{The data frame used in the call.}
\item{correct.AUC}{Logical value indicating whether bias-corrected AUC was used.}
\item{results}{A list containing estimated cut points, AUC and
bias-corrected AUC for each method.}
\item{control}{The control parameters used in the call.}
}
}
\description{
Returns an object with the optimal cut points to categorise a
continuous predictor variable in a logistic regression model
}
\examples{
library(CatPredi)
\dontrun{
set.seed(127)
#Simulate data
n = 100
#Predictor variable
xh <- rnorm(n, mean = 0, sd = 1)
xd <- rnorm(n, mean = 1.5, sd = 1)
x <- c(xh, xd)
#Response
y <- c(rep(0,n), rep(1,n))
#Covariate
zh <- rnorm(n, mean=1.5, sd=1)
zd <- rnorm(n, mean=1, sd=1)
z <- c(zh, zd)
# Data frame
df <- data.frame(y = y, x = x, z = z)

# Select optimal cut points using the AddFor algorithm
res.addfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 2,
                       data = df, method = "addfor", range=NULL, correct.AUC=FALSE,
                       control=controlcatpredi(grid=20))

# Select optimal cut points using the BackAddFor algorithm
res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
                           data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
}
\dontrun{
  set.seed(127)
  #Simulate data
  n = 200
  #Predictor variable
  xh <- rnorm(n, mean = 0, sd = 1)
  xd <- rnorm(n, mean = 1.5, sd = 1)
  x <- c(xh, xd)
  #Response
  y <- c(rep(0,n), rep(1,n))
  #Covariate
  zh <- rnorm(n, mean=1.5, sd=1)
  zd <- rnorm(n, mean=1, sd=1)
  z <- c(zh, zd)
  # Data frame
  df <- data.frame(y = y, x = x, z = z)

  # Select optimal cut points using the AddFor algorithm
  res.addfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
                         data = df, method = "addfor", range=NULL, correct.AUC=FALSE)

  # Select optimal cut points using the BackAddFor algorithm
  res.backaddfor <- catpredi(formula = y ~ z, cat.var = "x", cat.points = 3,
                             data = df, method = "backaddfor", range=NULL, correct.AUC=FALSE)
}

}
\references{
I Barrio, J Roca-Pardinas and I Arostegui (2021). Selecting the number
of categories of the lymph node ratio in cancer research: A bootstrap-based
hypothesis test. \emph{Statistical Methods in Medical Research}, 30(3), 926-940.

I Barrio, I Arostegui, M.X Rodriguez-Alvarez and  J.M Quintana (2017).
A new approach to categorising continuous variables in prediction models:
proposal and validation. \emph{Statistical Methods in Medical Research}, 26(6), 2586-2602.

S.N Wood (2006). Generalized Additive Models: An Introduction with R. Chapman and Hall/CRC.
}
\seealso{
\code{\link{controlcatpredi}},
\code{\link{comp.cutpoints}},
\code{\link{plot.catpredi}},
\code{\link{summary.catpredi}}
}
\author{
Irantzu Barrio, Maria Xose Rodriguez-Alvarez, Inmaculada Arostegui, Javier Roca-Pardinas and Xabier Amutxastegi.
}
